;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Primitives"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverIterations(4)
btContactSetSolverMode(SOLVER_SIMD)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,100,-260
RotateEntity camera,25,0,0

light=CreateLight(1)
PositionEntity light,0,100,0

texPlane=LoadTexture("..\media\cell.png")
ScaleTexture texPlane,20,20
plane=CreatePlane()
EntityTexture  plane,texPlane

btBodyCreatePlane(0,1,0)

Global hide=0
Global capsule=CreateCapsule(29,4)

For i=0 To 2
	CreatePrimitive(0)
	CreatePrimitive(1)
	CreatePrimitive(2) 
	CreatePrimitive(3)
	CreatePrimitive(4)
Next

Repeat	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	If KeyHit(50) Then For i=0 To 4 CreatePrimitive(0) Next
	If KeyHit(49) Then For i=0 To 4 CreatePrimitive(1) Next
	If KeyHit(48) Then For i=0 To 4 CreatePrimitive(2) Next
	If KeyHit(47) Then For i=0 To 4 CreatePrimitive(3) Next
	If KeyHit(46) Then For i=0 To 4 CreatePrimitive(4) Next

	If KeyHit(35) Then HideShowMeshes()
	
	time=MilliSecs ()
	btSimulation(0.1,2)
	New_time=MilliSecs ()-time

	UpdateWorld
	RenderWorld()
	btDebugDrawWorld(camera)

	frames=frames+1  
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"dll time: "+New_time
	
	Text 10,50,"1,2,3,4 - debug draw mode"
	Text 10,70,"H - hide/show meshes"
	Text 10,90,"M - create capsule"
	Text 10,110,"N - create box"
	Text 10,130,"B - create sphere"
	Text 10,150,"V - create cone"
	Text 10,170,"N - create cylinder"
	Text 10,190,"Count body: "+btGetNumBodies()
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()

End

Type phEntity
	Field mesh
	Field body
End Type

Function CreatePrimitive(pType)

	phEnt.phEntity = New phEntity
	Select pType
	Case 0
		phEnt\mesh=CopyEntity(capsule)
		phEnt\body=btBodyCreateCapsule(4,24,Rnd(10,50))
	Case 1
		dx#=Rnd(5,8)
		dy#=Rnd(5,8)
		dz#=Rnd(5,8)
		phEnt\mesh=CreateCube()
		ScaleEntity phEnt\mesh,dx,dy,dz
		phEnt\body=btBodyCreateBox(dx,dy,dz,Rnd(10,50))
	Case 2
		radius#=Rnd(7,12)
		phEnt\mesh=CreateSphere()
		ScaleEntity phEnt\mesh,radius,radius,radius
		phEnt\body=btBodyCreateSphere(radius,Rnd(10,50))
	Case 3
		radius#=Rnd(7,12)
		height#=Rnd(12,20)
		phEnt\mesh=CreateCone()
		ScaleEntity phEnt\mesh,radius,height,radius
		phEnt\body=btBodyCreateCone(radius,height,Rnd(10,50))
	Case 4
		radius#=Rnd(4,6)
		height#=Rnd(14,35)
		phEnt\mesh=CreateCylinder()
		ScaleEntity phEnt\mesh,radius,height,radius
		phEnt\body=btBodyCreateCylinder(radius,height,Rnd(10,50))
	End Select 
	If hide=1
		HideEntity phEnt\mesh
	EndIf

	EntityColor phEnt\mesh,Rand(100,200),Rand(100,200),Rand(100,200)
	btBodySetPosition(phEnt\body,Rnd(-150,150),Rnd(50,100),Rnd(-150,150))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)

	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)
End Function

Function HideShowMeshes()
	
	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh	
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
		Next
		hide=0
	EndIf
	
End Function

Function CreateCapsule(height#, radius#)

capsule= CreateCylinder()
ScaleMesh capsule, radius, height/2, radius
PositionMesh capsule,0,0,0

sphere1 = CreateSphere() 
ScaleMesh sphere1,radius,radius,radius
PositionMesh sphere1,0, height/2,0
AddMesh sphere1,capsule

sphere2 = CreateSphere() 
ScaleMesh sphere2,radius,radius,radius
PositionMesh sphere2,0, -height/2,0
AddMesh sphere2,capsule

FreeEntity sphere1
FreeEntity sphere2
HideEntity capsule

Return capsule

End Function
