;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Trigger"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)

camera=CreateCamera()
CameraClsColor camera,20,20,20
CameraRange  camera,1,2500
AmbientLight 140,140,140
PositionEntity camera,0,100,-160
RotateEntity camera,25,0,0

AmbientLight 140,140,140
light=CreateLight()
PositionEntity light,0,100,-250
RotateEntity  light,60,0,0

mirror=CreateMirror()
Global plane=CreatePlane()
EntityAlpha plane,0.6
texPlane=LoadTexture( "..\media\cell.png" )
ScaleTexture texPlane,50,50
EntityTexture plane,texPlane,0,1

bodyPlane=btBodyCreatePlane(0,1,0)
btBodySetFriction(bodyPlane,0.08)
btBodySetContactProcessingThreshold(bodyPlane,0.032)
btBodySetMargin(bodyPlane,0.2)
btBodySetPosition(bodyPlane,0,-1,0)
btBodyEnableTriggertable(bodyPlane,0)

Global bodyTrigger=btBodyCreateBox(140,7,10,0.5)

btBodySetActivationState(bodyTrigger,DISABLE_DEACTIVATION)
btBodySetLinearFactorX(bodyTrigger,0)
btBodySetLinearFactorY(bodyTrigger,0)
btBodySetLinearFactorZ(bodyTrigger,0)
btBodySetGravity(bodyTrigger,0,0,0)

btBodyEnableCollision(bodyTrigger,0)
btBodySetPosition(bodyTrigger,0,9,0)
btBodySetRotation(bodyTrigger,0,40,0)
btBodyEnableRayCast(bodyTrigger,0)
btIntersectionTestAddTrigger(bodyTrigger)

Global entTrigger=CreateCube()
ScaleEntity entTrigger,140,7,10
EntityColor entTrigger,10,10,250
EntityAlpha entTrigger,0.4
btBodyAttachEntity(bodyTrigger,entTrigger)

Const CollisionGroup1=1
Const CollisionGroup2=2

btMask=btCreateMask()
btMaskAddGroup(btMask,CollisionGroup2)

ray=btCreateRay()
btRaySetPosition(ray,0,0,0)
btRaySetDirection(ray,0,1,0)
btRaySetMaxDistance(ray,10)


For i=0 To 100
	CreateBox(Rnd(-90,90),Rnd(90,150),Rnd(-90,90),Rand(CollisionGroup1,CollisionGroup2))
Next

For i=0 To 5
	CreateStaticBox(Rnd(-90,90),Rnd(9,10),Rnd(-90,90))
Next

Const UPS=60
period=1000/UPS
time=MilliSecs()-period

methodTest=0
modeTestBody=0
mask=0
modeName$="(all bodies)"
methodCheck$="(fast test)"

Repeat

	Repeat
		elapsed=MilliSecs()-time
	Until elapsed
	ticks=elapsed/period
	
	For k=1 To ticks
		time=time+period
		
		If KeyHit(2) btDebugDrawSetMode(0)
		If KeyHit(3) btDebugDrawSetMode(1)
		If KeyHit(4) btDebugDrawSetMode(2)
		If KeyHit(5) btDebugDrawSetMode(1+2)

		If KeyHit(50) methodTest=0 : methodCheck$="(fast test)"
		If KeyHit(49) methodTest=1 : methodCheck$="(manifolds)" 
		If KeyHit(48) methodTest=2 : methodCheck$="(bounding box)" 

		If KeyHit(37) modeTestBody=0 : modeName$="(all bodies)"
		If KeyHit(36) modeTestBody=1 : modeName$="(only dynamic)" 
		If KeyHit(35) modeTestBody=2 : modeName$="(only static)" 

		If  KeyHit(25) 
			If mask=0
				mask=btMask
			Else
				mask=0
			EndIf
		EndIf
	
		btBodySetAngularVelocity(bodyTrigger,0,0.2,0)

		btSimulation(0.2,4)
		
		btTriggerSetIntersectionParam(bodyTrigger,methodTest,modeTestBody,mask)
		btTriggersIntersectionTest(False)
		CheckBox()	
		UpdateWorld
	Next
	
	RenderWorld()
	HideWorld()
	btDebugDrawWorld(camera)
	ShowWorld()

	Frames=Frames+1  
   	If MilliSecs()-render_time=>1000  fps1=Frames : Frames=0 : render_time=MilliSecs()  
	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"	
	Text 10,50,"M, N, B - method check "+methodCheck
	Text 10,70,"K, J, H - mode of check a body "+modeName
	If mask=0
		Text 10,90,"P - use a mask "
	Else
		Text 10,90,"P - not use a mask "
	EndIf
	
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()

End

Type btBody
	Field body
	Field entity
	Field isStatic
	Field group
End Type

Function CreateBox(x#,y#,z#,setGroup%)
	b.btBody=New btBody
	dx#=Rnd(2,6)
	dy#=Rnd(2,6)
	dz#=Rnd(2,6)
	b\body=btBodyCreateBox(dx,dy,dz,Rnd(5,15))
	btBodySetPosition(b\body,x,y,z)
	b\entity=CreateCube()
	b\group=setGroup
	If b\group=1
		EntityColor b\entity,50,200,50
	Else
		EntityColor b\entity,200,170,50
	EndIf
	ScaleEntity  b\entity,dx,dy,dz
	btBodyAttachEntity(b\body,b\entity)
	btBodySetCollisionGroup(b\body,setGroup)
	b\isStatic=False
End Function

Function CreateStaticBox(x#,y#,z#)
	b.btBody=New btBody
	dx#=Rnd(2,2)
	dy#=Rnd(12,14)
	dz#=Rnd(2,3)
	b\body=btBodyCreateBox(dx,dy,dz,0)
	btBodySetPosition(b\body,x,y,z)
	b\entity=CreateCube()
	ScaleEntity  b\entity,dx,dy,dz
	btBodyAttachEntity(b\body,b\entity)
	b\isStatic=True
End Function

Function CheckBox()

	For b.btBody=Each btBody
		If(btTriggerIsBodyIntersect(bodyTrigger,b\body))
			EntityColor b\entity,240,110,110
		Else
			If b\isStatic=True
				EntityColor b\entity,40,40,180	
			Else
				If b\group=1
					EntityColor b\entity,50,200,50
				Else
					EntityColor b\entity,200,170,50
				EndIf
			EndIf
		EndIf
	Next
	
End Function 

Function HideWorld()

	HideEntity plane
	HideEntity entTrigger
	For b.btBody=Each btBody	
		HideEntity b\entity
	Next
	
End Function 

Function ShowWorld()

	ShowEntity plane
	ShowEntity entTrigger
	For b.btBody=Each btBody	
		ShowEntity b\entity
	Next
	
End Function 
