;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Magnet"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverIterations(4)

camera=CreateCamera()
CameraClsColor camera,20,20,20
CameraRange  camera,1,2500
AmbientLight 140,140,140
PositionEntity camera,0,100,-160
RotateEntity camera,25,0,0

light=CreateLight(1)
PositionEntity light,-50,150,-260
RotateEntity  light,0,0,0

mirror=CreateMirror()
PositionEntity mirror,0,1,0
texPlane=LoadTexture("..\media\cell.png")
ScaleTexture texPlane,20,20
plane=CreatePlane()
EntityTexture  plane,texPlane
EntityAlpha plane,0.8

bodyPlane=btBodyCreatePlane(0,1,0)
btBodySetFriction(bodyPlane,0.08)
btBodySetContactProcessingThreshold(bodyPlane,0.032)
btBodySetMargin(bodyPlane,0.2)

Global meshMagnet=LoadMesh("..\media\magnet.b3d")
EntityFX meshMagnet,1
bodyMagnet=btBodyCreateConvexHull(meshMagnet,0,1)
btBodyAttachEntity(bodyMagnet, meshMagnet)
btBodySetGravity(bodyMagnet,0,0,0)
btBodySetPosition(bodyMagnet,0,50,0)
btBodySetRotation(bodyMagnet,0,0,180)
btBodyEnableCollision(bodyMagnet,0)

magnet=btCreateMagnet(17,0.3)
btMagnetSetBody(magnet,bodyMagnet)
btMagnetSetMaxRadius(magnet,100)
btMagnetSetMinRadius(magnet,28)
btMagnetSetStepUpdate(magnet,2)

maskRed=btCreateMask()
btMaskAddGroup(maskRed,1)
btMaskAddGroup(maskRed,2)

maskGreen=btCreateMask()
btMaskAddGroup(maskGreen,1)
btMaskAddGroup(maskGreen,3)

Global hide=0
For i=0 To 80
	phCreateSphere(1) 
Next
For i=0 To 50
	phCreateSphere(2) ;red
Next
For i=0 To 50
	phCreateSphere(3) ;green
Next

Const UPS=60
period=1000/UPS
time=MilliSecs()-period

Repeat
	
	Repeat
		elapsed=MilliSecs()-time
	Until elapsed
	ticks=elapsed/period
	
	For k=1 To ticks
		time=time+period
		
		If KeyHit(2) btDebugDrawSetMode(0)
		If KeyHit(3) btDebugDrawSetMode(1)
		If KeyHit(4) btDebugDrawSetMode(2)
		If KeyHit(5) btDebugDrawSetMode(1+2)
			
		If KeyDown(200) btBodyTranslate(bodyMagnet,0,0,1.5)
		If KeyDown(208) btBodyTranslate(bodyMagnet,0,0,-1.5)	
		If KeyDown(205) btBodyTranslate(bodyMagnet,1.5,0,0)
		If KeyDown(203) btBodyTranslate(bodyMagnet,-1.5,0,0)
							
		If KeyHit(50) Then btMagnetSetMask(magnet,maskRed)
		If KeyHit(49) Then btMagnetSetMask(magnet,maskGreen)
		If KeyHit(48) Then btMagnetSetMask(magnet,0)
													
		If KeyHit(35) Then HideShowMeshes()
		
		btSimulation(0.1,3)
		UpdateWorld
	Next
	
	RenderWorld()
	btDebugDrawWorld(camera)

	Frames=Frames+1  
   	If MilliSecs()-render_time=>1000  fps1=Frames : Frames=0 : render_time=MilliSecs()  
	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"	
	Text 10,50,"Arrows - move magnet"
	Text 10,70,"H - hide/show meshes"
	Text 10,90,"M - use a mask (red only)"
	Text 10,110,"N - use a mask (green only)"
	Text 10,130,"B - not use a mask (all)"
	
	Flip 0
	
Until KeyHit(1)

btDestroyWorld()

End


Type phEntity
	Field mesh
	Field body
End Type

Function phCreateSphere(group)
	
	phEnt.phEntity = New phEntity
	radius#=Rnd(4,6)
	phEnt\mesh=CreateSphere()
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,Rnd(5,10))
			
	If hide=1
		HideEntity phEnt\mesh
	EndIf
	
	Select group
		Case 1
			EntityColor phEnt\mesh,Rand(100,200),Rand(100,200),Rand(100,200)
			btBodySetCollisionGroup(phEnt\body,1)
		Case 2
			EntityColor phEnt\mesh,20,150,20
			btBodySetCollisionGroup(phEnt\body,2)
		Case 3
			EntityColor phEnt\mesh,150,20,20
			btBodySetCollisionGroup(phEnt\body,3)
	End Select
	
	btBodySetPosition(phEnt\body,Rnd(-150,150),Rnd(50,100),Rnd(-150,150))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	
	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)
End Function

Function HideShowMeshes()
	
	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh
			HideEntity meshMagnet		
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
			ShowEntity meshMagnet
		Next
		hide=0
	EndIf
	
End Function
