;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Joints"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(DBG_NAMEJOINT+DBG_JOINTS+DBG_JOINTLIMITS)
btWorldSetFixedTimeStep(1.0/60)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,90,-85
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,25,0,20

Global red=120,green=120,blue=250
Global meshPlane=CreatePlane()
Global tex=LoadTexture("..\media\cell.png")
Global tex_mesh=LoadTexture("..\media\cell.png")
ScaleTexture tex,10,10
EntityTexture meshPlane,tex,0,1 

body=phCreateCube(12,25,1)
btBodySetPosition(body,-60,50,0)

joint_Hinge=btJointCreateHinge(body,0, -20,0,0,0, 0,0,0,0, 0)
btJointHingeSetLimit(joint_Hinge,-90,90)
btJointHingeSetAxis(joint_Hinge,0,-1,0)

body=phCreateCube(5,5,5)
btBodySetPosition(body,-30,40,0)

joint_Slider=btJointCreateSlider(body,0, 0,0,0,0, 0,0,0,0, 0)
btJointSliderSetLinearLimit(joint_Slider,-40,40)
btJointSetRotationA(joint_Slider,0,90,0)
btJointSetRotationB(joint_Slider,0,90,0)

body=phCreateCube(5,5,5)
btBodySetPosition(body,0,60,0)

joint_6Dof=btJointCreate6DofGeneric(body,0, 0,20,0,0, 0,0,0,0, 0)
btJoint6DofSetAngularLowerLimit(joint_6Dof,-30,-30,-30)
btJoint6DofSetAngularUpperLimit(joint_6Dof,30,30,30)
btJointSetRotationA(joint_6Dof,0,0,90)
btJointSetRotationB(joint_6Dof,0,0,90)

body=phCreateCube(5,5,5)
btBodySetPosition(body,30,50,0)

joint_Point2Point=btJointCreatePoint2Point(body,0, 0,20,0,0, 0,0,0,0, 0)

body0=phCreateCube(5,5,5)
btBodySetPosition(body0,60,40,0)

body1=btBodyCreateBox(2,2,2,0)
btBodySetPosition(body1,60,70,0)

joint_ConeTwist=btJointCreateConeTwist(body0,body1, 0,15,0,0, 0,-15,0,0, 0)
btJointConeTwistSetLimit(joint_ConeTwist,30,30,30)
btJointSetRotationB(joint_ConeTwist,0,0,90)
btJointSetRotationA(joint_ConeTwist,0,0,90)

btRay=btCreateRay()
btRaySetMaxDistance(btRay,500)

entity=0
Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

Repeat	
	
	If KeyHit(2) btDebugDrawSetMode(DBG_NAMEJOINT+DBG_JOINTS+DBG_JOINTLIMITS)
	If KeyHit(3) btDebugDrawSetMode(DBG_NAMEJOINT+DBG_WF+DBG_JOINTLIMITS+DBG_JOINTS)
	If KeyHit(4) btDebugDrawSetMode(DBG_NAMEJOINT+DBG_AABB+DBG_JOINTLIMITS+DBG_JOINTS)
	If KeyHit(5) btDebugDrawSetMode(DBG_NAMEJOINT+DBG_WF+DBG_AABB+DBG_JOINTLIMITS+DBG_JOINTS)

	If KeyHit(35) Then HideShowMeshes()
	
	msx#=MouseXSpeed()
	msy#=MouseYSpeed()
	Camera_Look(camera)

	btRaySetPosition(btRay,EntityX(camera,1),EntityY(camera,1),EntityZ(camera,1))
	
	mouse_hit=MouseHit(1) 
	
	CameraPick(camera,MouseX(),MouseY())
	If PickedEntity()<>0
		entity=PickedEntity()
		EntityColor entity,150,150,220
		If mouse_hit=1 
			btRaySetDirection(btRay,PickedX#()-EntityX(camera,1), PickedY#()-EntityY(camera,1), PickedZ#()-EntityZ(camera,1))
			btRayGetPickInfo(btRay,0)
			If(btPickInfoBody()<>0)
				btBodyApplyForce(btPickInfoBody(),btPickInfoNX()*-1000,btPickInfoNY()*-1000,btPickInfoNZ()*-1000)
			EndIf
		EndIf
	Else
		If entity<>0 Then EntityColor entity,red,green,blue
	EndIf
	
	btSimulation(.1,6)
	
	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)
	
	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	Text 10,70,"Click on entity - apply force"

	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
	Field group
End Type

Function phCreateSphere%()
	
	phEnt.phEntity = New phEntity
	radius#=Rnd(4,6)
	phEnt\mesh=CreateSphere()
	EntityPickMode phEnt\mesh,2
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,Rnd(5,10))
			
	If hide=1
		EntityAlpha phEnt\mesh,0
	EndIf

	EntityColor phEnt\mesh,red,green,blue
	
	btBodySetPosition(phEnt\body,Rnd(-50,150),Rnd(50,100),Rnd(-50,50))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	
	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)

	Return phEnt\body
End Function

Function phCreateCube(dx#, dy#, dz#, mass#=10)
	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateCube()
	EntityPickMode phEnt\mesh,2
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,dx,dy,dz
	phEnt\body=btBodyCreateBox(dx,dy,dz,mass)
	EntityColor phEnt\mesh,red,green,blue
	;ScaleTexture tex_mesh,1,1
	;EntityTexture phEnt\mesh,tex_mesh,0,1 
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.02)
	btBodySetLinearDamping(phEnt\body,0.2)
	
	btBodySetAngularSleepingThreshold(phEnt\body,0.08)
	btBodySetLinearSleepingThreshold(phEnt\body,0.06)

	Return phEnt\body
End Function

Function phSphereGetEntity%()
	
	phEnt.phEntity = Last phEntity
	Return phEnt\mesh
	
End Function

Function HideShowMeshes()

	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			EntityAlpha phEnt\mesh,0
			HideEntity meshPlane
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			EntityAlpha phEnt\mesh,1
			ShowEntity meshPlane
		Next
		hide=0
	EndIf
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)

	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function

Function ActiveCamera(cam,pivot_car,ent_car,cspeedX#,cspeedY#,cspeedZ#,dist#,hite#,xrot#,Modecamera)

	If KeyDown (30) Then 
		TurnEntity pivot_car,0,-1,0
	Else	
		If KeyDown (32) Then TurnEntity pivot_car,0,1,0
 	EndIf
	
	Select Modecamera	  	    
		Case 0
		    EntityParent cam,0
			PointEntity cam,pivot_car
			
			TFormPoint 0,hite,dist,pivot_car,0
			cx#=(TFormedX()-EntityX(cam))*(cspeedX#)
			cy#=(TFormedY()-EntityY(cam))*(cspeedY#)
			cz#=(TFormedZ()-EntityZ(cam))*(cspeedZ#)
			TranslateEntity cam,cx,cy,cz
			flagPosCamera=0
		Case 2		 
		    RotateEntity cam,0,0,0
			PositionEntity cam,0,35,15
			EntityParent cam,ent_car	
			flagPosCamera=0						
		Case 3
		    EntityParent cam,ent_car
			RotateEntity cam,0,0,0
			PositionEntity cam,-10,27,-4
			flagPosCamera=0	
		Case 4
			RotateEntity cam,0,0,0
			PositionEntity cam,-27,15,-2
			flagPosCamera=0	
		Case 1
			EntityParent cam,0
			If flagPosCamera=0
				PositionEntity cam,400,170,-260
				flagPosCamera=1
			EndIf
			PointEntity cam,ent_car	   	    	
	End Select 
	
End Function
