;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Raycast - Advanced"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(DBG_RAYS+DBG_RAYCASTPOINTS)
btDebugDrawSetSizePointRays(3)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverMode(SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,100,-150
RotateEntity camera,20,0,0
CameraClsColor camera,40,40,40

AmbientLight 150,150,150
light=CreateLight()
PositionEntity light,0,400,0
RotateEntity light,10,0,0

Global groundMesh=CreatePlane()
EntityColor groundMesh,100,100,100
tex=LoadTexture("..\media\cell.png")
ScaleTexture tex,40,40
EntityTexture groundMesh,tex,0,1 

groundBody=btBodyCreateBox(500,1,500,0)
btBodySetMargin(groundBody,0.1)

tex_grey=LoadTexture("..\media\cell_grey.png")
ScaleTexture tex_grey,0.5,0.5

Global box=CreateCube()
ScaleEntity box,10,10,10
EntityColor box,100,100,100
EntityTexture box,tex_grey

bodyBox=btBodyCreateBox(10,10,10,5)
btBodyAttachEntity(bodyBox, box)
btBodySetPosition(bodyBox, 80,20,-40)


Global chair=LoadMesh("..\media\stool.b3d")
ScaleMesh chair,2,1.8,2
EntityColor chair,100,100,100
EntityTexture chair,tex_grey

bodyChair=btBodyCreateTrimesh(chair,5,2)
btBodyAttachEntity(bodyChair, chair)
btBodySetPosition(bodyChair, 0,20,0)


Global torus=LoadMesh("..\media\torus.b3d")
ScaleMesh torus,0.5,0.5,0.5
EntityColor torus,250,250,250
surf=GetSurface(torus,1)
brush=GetSurfaceBrush(surf)
BrushColor brush,100,100,100
PaintSurface surf,brush 
EntityTexture torus,tex_grey,0,0

softBody=btSoftBodyCreateFromMesh(torus,10,2)
btBodySetRotation(softBody,0,0,0)
btBodySetPosition(softBody,-80,10,30)
btBodyAttachEntity(softBody,torus)
btSoftBodySetFlagCollisions(softBody,SC_CLRS)
btSoftBodyGenerateClusters(softBody,16)
btSoftBodySetLift(softBody,0.2)

Ray=btCreateRay()
btRaySetMethodPick(Ray,2)
btRaySetPosition(Ray,110,8,-60)
btRaySetMaxDistance(Ray,10)


Global mx#,my#,msx#,msy#
Global hide=0
Const UPS=60
period=1000/UPS
time=MilliSecs()-period

posx#=0
Repeat

	If KeyHit(2) btDebugDrawSetMode(DBG_RAYS+DBG_RAYCASTPOINTS)
	If KeyHit(3) btDebugDrawSetMode(1+DBG_RAYS+DBG_RAYCASTPOINTS)
	If KeyHit(4) btDebugDrawSetMode(2+DBG_RAYS+DBG_RAYCASTPOINTS)
	If KeyHit(5) btDebugDrawSetMode(1+2+DBG_RAYS+DBG_RAYCASTPOINTS)

	If KeyHit(35) Then HideShowMeshes()

	msx#=MouseXSpeed()
	msy#=MouseYSpeed()

	j#=j+0.2
	posx#=posx+(Cos(j)*0.27)
	
	btRaySetPointDir(Ray,-280*(Cos(posx)),8,80)
	btRayGetPickInfo(Ray,0)

	UpdateWorld
	btSimulation(0.09,6)
	
	RenderWorld()
	btDebugDrawWorld(camera)
	Frames=Frames+1  
   	If MilliSecs()-render_time=>1000  fps1=Frames : Frames=0 : render_time=MilliSecs()
	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	For i=0 To btPickInfoCount()-1
		CameraProject camera,btPickInfoX(0,i),btPickInfoY(0,i),btPickInfoZ(0,i)
		Text ProjectedX(),ProjectedY(),btPickInfoDistance (0,i)
		bodyType=btBodyGetType(btPickInfoBody(0,i))
		If bodyType=11
			Text ProjectedX(),ProjectedY()-20,"Triangle: "+btPickInfoTriangle(0,i)
		Else
			If bodyType=12 Then Text ProjectedX(),ProjectedY()-20,"Node: "+btPickInfoNode(0,i)
		EndIf
	Next

	Flip 1
	
Until KeyHit(1)
btDestroyWorld()
End

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)
	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function

Function HideShowMeshes()
	
	If hide=0 Then
		HideEntity box
		HideEntity groundMesh		
		HideEntity chair
		HideEntity torus
		hide=1
	Else
		ShowEntity box
		ShowEntity groundMesh		
		ShowEntity chair
		ShowEntity torus
		hide=0
	EndIf
	
End Function
