;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Collision group"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,140,-120
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,25,0,20

Global meshPlane=CreatePlane()
texPlane=LoadTexture( "..\media\cell.png" )
ScaleTexture texPlane,40,40
EntityTexture meshPlane,texPlane,0,1 

btBodyCreatePlane(0,1,0)

Const GroupRed=2
Const GroupGreen=3
Const GroupBlue=4
Const GroupYellow=5

For i=1 To 12
	phCreateCube(5,60-i,i*10-5,0,GroupRed)
Next

For i=1 To 12
	phCreateCube(5,20-i,i*10-5,0,GroupGreen)
Next

For i=1 To 12
	phCreateCube(5,-20+i,i*10-5,0,GroupBlue)
Next

For i=1 To 12
	phCreateCube(5,-60+i,i*10-5,0,GroupYellow)
Next

btCollisionGroupSetFlag(GroupRed,GroupGreen,0)
btCollisionGroupSetFlag(GroupBlue,GroupGreen,0)
btCollisionGroupSetFlag(GroupRed,GroupYellow,0)
btCollisionGroupSetFlag(GroupBlue,GroupYellow,0)



Global hide=0
Repeat	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	If KeyHit(35) Then HideShowMeshes()

	If KeyHit(57)
		i=0
		For phEnt.phEntity = Each phEntity
			i=i+1
			If i<=12
				btBodySetPosition phEnt\body,60-i,i*10-5,0
			Else
				If i>12 And i<=24
					btBodySetPosition phEnt\body,20-(i-12),(i-12)*10-5,0
				Else
					If i>24 And i<=36
						btBodySetPosition phEnt\body,-20+(i-24),(i-24)*10-5,0
					Else
						btBodySetPosition phEnt\body,-60+(i-36),(i-36)*10-5,0
					EndIf
				EndIf
			EndIf
			btBodySetRotation(phEnt\body,0,0,0)
			btBodySetAngularVelocity(phEnt\body,0,0,0)
			btBodySetLinearVelocity(phEnt\body,0,0,0)
		Next
	EndIf

	btSimulation(.06,3)

	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)

	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	Text 10,70,"Space - reset scene"

	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
	Field group
End Type

Function phCreateCube(size#, x#, y#, z#, group)
	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateCube()
	EntityShininess phEnt\mesh,0.5
	dx#=size
	dy#=size
	dz#=size
	ScaleEntity phEnt\mesh,dx,dy,dz
	phEnt\body=btBodyCreateBox(dx,dy,dz,Rnd(2,4))	
	btBodySetPosition(phEnt\body,x,y,z)
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	phEnt\group=group
	
	Select group
	Case 2
		EntityColor phEnt\mesh,Rnd(170,190),Rnd(120,100),Rnd(120,100)	
		btBodySetCollisionGroup(phEnt\body,group)
	Case 3
		EntityColor phEnt\mesh,Rnd(120,100),Rnd(170,190),Rnd(120,100)
		btBodySetCollisionGroup(phEnt\body,group)
	Case 4
		EntityColor phEnt\mesh,Rnd(120,100),Rnd(120,100),Rnd(170,190)
		btBodySetCollisionGroup(phEnt\body,group)
	Case 5
		EntityColor phEnt\mesh,Rnd(220,200),Rnd(180,210),Rnd(90,100)
		btBodySetCollisionGroup(phEnt\body,group)
	Default
		btBodySetCollisionGroup(phEnt\body,group)
	End Select
	
	Return phEnt\body
End Function

Function HideShowMeshes()

	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh
			HideEntity meshPlane
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
			ShowEntity meshPlane
		Next
		hide=0
	EndIf
	
End Function
