;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Collision detection"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(8)
btDebugDrawSetSizeContacts(12)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,100,-250
RotateEntity camera,20,0,0
CameraRange camera,1,2500

AmbientLight 150,150,150
light=CreateLight()
PositionEntity light,0,400,0
RotateEntity light,10,0,0

Global plane=CreatePlane()
texPlane=LoadTexture( "..\media\cell.png" )
ScaleTexture texPlane,25,25
EntityTexture plane,texPlane,0,1 

bodyPlane=btBodyCreatePlane(0,1,0)
btBodySetRestitution(bodyPlane,0.0)
btBodyEnableDebugDraw(bodyPlane,0)

CreateListener(camera,0.005)
soundContact=Load3DSound("..\media\bounce.wav")

For i=0 To 10
	phCreateCube(Rand(12,16),Rnd(90,140))
	phCreateSphere()
Next

controlBody=phCreateCube(18,0)
btBodySetPosition(controlBody,0,20,-80)
btBodyEnableCollision(controlBody,0)
btBodyEnableResponse(controlBody,1)
EntityAlpha phGetLastEntity(),0.5

btBodyEnableDebugDraw(controlBody,1)


Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

Repeat

	msx#=MouseXSpeed()
	msy#=MouseYSpeed()

	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1+8)
	If KeyHit(4) btDebugDrawSetMode(1+2+8)
	If KeyHit(5) btDebugDrawSetMode(8)

	If KeyHit(35) Then HideShowMeshes()

	btBodyTranslate(controlBody,msx*0.3,0,-msy*0.3)

	CameraProject camera,btBodyGetX(controlBody),btBodyGetY(controlBody),btBodyGetZ(controlBody)
	MoveMouse ProjectedX(),ProjectedY()
	
	btSimulation(0.3,7)
	UpdateWorld()

	RenderWorld()
	btDebugDrawWorld(camera)
	Frames=Frames+1
	
   	If MilliSecs()-render_time=>1000  fps1=Frames : Frames=0 : render_time=MilliSecs()
	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	Text 10,70,"Mouse - body movement"

	btContactTestAddBody(controlBody,0)
	btContactManifoldTest(True)

	contacts=btContactGetNum(controlBody)
	typeBody$="none"
	If contacts>0 Then
		If btBodyGetType((btContactGetBody(controlBody,0)))=3
			typeBody$="sphere"
		Else
			typeBody$="cube"
		EndIf
	EndIf
	
	Text 10,110,"Count contacts: "+contacts
	Text 10,130,"Type body: "+typeBody

	Flip 1
	
Until KeyHit(1)
btDestroyWorld()
End

Type phEntity
	Field mesh
	Field mass#
	Field body
End Type

Function phCreateSphere%()
	phEnt.phEntity = New phEntity
	radius#=16
	phEnt\mesh=CreateSphere()
	If hide=1 Then HideEntity phEnt\mesh
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\mass=Rnd(10,14)
	phEnt\body=btBodyCreateSphere(radius,phEnt\mass)
	EntityColor phEnt\mesh,Rand(100,150),Rand(100,150),Rand(100,150)		
	btBodySetPosition(phEnt\body,Rnd(-250,250),Rnd(50,290),Rnd(-250,250))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.02)
	btBodySetLinearDamping(phEnt\body,0.2)
	
	btBodySetAngularSleepingThreshold(phEnt\body,0.6)
	btBodySetLinearSleepingThreshold(phEnt\body,0.6)
	
	Return phEnt\body
End Function

Function phCreateCube%(size%=5, mass#=10)
	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateCube()
	If hide=1 Then HideEntity phEnt\mesh
	dx#=size
	dy#=size
	dz#=size
	ScaleEntity phEnt\mesh,dx,dy,dz
	phEnt\mass=mass
	phEnt\body=btBodyCreateBox(dx,dy,dz,phEnt\mass)
	EntityColor phEnt\mesh,Rnd(120,180),Rnd(120,180),Rnd(120,180)		
	btBodySetPosition(phEnt\body,Rnd(-150,150),Rnd(250,400),Rnd(-150,150))
	;btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.25)
	btBodySetLinearDamping(phEnt\body,0.1)
	btBodySetAngularDamping(phEnt\body,0.2)

	btBodyEnableDebugDraw(phEnt\body,1)

	
	btBodySetAngularSleepingThreshold(phEnt\body,0.4)
	btBodySetLinearSleepingThreshold(phEnt\body,0.6)
	
	Return phEnt\body
End Function

Function phGetLastEntity%()

	phEnt.phEntity = Last phEntity
	Return phEnt\mesh
	
End Function

Function HideShowMeshes()
	
	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh
			HideEntity plane		
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
			ShowEntity plane
		Next
		hide=0
	EndIf
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)
	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function
