;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Compound"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,100,-120
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,25,0,20

Global meshPlane=CreatePlane()
texPlane=LoadTexture( "..\media\cell.png" )
ScaleTexture texPlane,40,40
EntityTexture meshPlane,texPlane,0,1 

bodyPlane=btBodyCreatePlane(0,1,0)

Global meshMainChair=CreateCube()
EntityColor meshMainChair,100,120,70
ScaleEntity meshMainChair,22,1,25
PositionEntity meshMainChair,0,0,0

pivotMainChair=CreatePivot()


bodyMainChair=btBodyCreateBox(22,1,25,0)
btBodySetPosition(bodyMainChair,0,0,0)

bodyChairLegLeft_1=phCreateChairElement(meshMainChair,2,25,2,-19,-25,-23)
bodyChairLegLeft_2=phCreateChairElement(meshMainChair,2,25,2,-19,-25,23)

bodyChairLegRight_1=phCreateChairElement(meshMainChair,2,25,2,19,-25,-23)
bodyChairLegRight_2=phCreateChairElement(meshMainChair,2,25,2,19,-25,23)

bodyChairBack_1=phCreateChairElement(meshMainChair,2,20,1,19,20,23)
bodyChairBack_2=phCreateChairElement(meshMainChair,2,20,1,-19,20,23)
bodyChairBack_3=phCreateChairElement(meshMainChair,24,6,1,0,40,23)

shapeCompound=btShapeCreateCompound()
btShapeCompoundAddBody(shapeCompound,bodyMainChair,0,1)
btShapeCompoundAddBody(shapeCompound,bodyChairLegLeft_1,0,1)
btShapeCompoundAddBody(shapeCompound,bodyChairLegLeft_2,0,1)
btShapeCompoundAddBody(shapeCompound,bodyChairLegRight_1,0,1)
btShapeCompoundAddBody(shapeCompound,bodyChairLegRight_2,0,1)
btShapeCompoundAddBody(shapeCompound,bodyChairBack_1,0,1)
btShapeCompoundAddBody(shapeCompound,bodyChairBack_2,0,1)
btShapeCompoundAddBody(shapeCompound,bodyChairBack_3,0,1)

bodyCompound=btBodyCreateCompound(shapeCompound,100)
btBodySetPosition(bodyCompound,0,80,0)

ent=-1
Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
Repeat	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	If KeyHit(35) Then HideShowMeshes()

	msx#=MouseXSpeed()
	msy#=MouseYSpeed()
	Camera_Look(camera)

	PositionEntity meshMainChair,btBodyGetX(bodyCompound),btBodyGetY(bodyCompound),btBodyGetZ(bodyCompound)
	RotateEntity meshMainChair,btBodyGetPitch(bodyCompound),btBodyGetYaw(bodyCompound),btBodyGetRoll(bodyCompound)

	PositionEntity pivotMainChair,btBodyGetX(bodyCompound),btBodyGetY(bodyCompound),btBodyGetZ(bodyCompound)
	RotateEntity pivotMainChair,btBodyGetPitch(bodyCompound),btBodyGetYaw(bodyCompound),btBodyGetRoll(bodyCompound)

	If MouseHit(1)
		sphere=phCreateSphere()
		TFormVector (0,0,1,camera,0)
		btBodySetPosition sphere,EntityX(camera)+TFormedX()*10,EntityY(camera)+TFormedY()*10,EntityZ(camera)+TFormedZ()*10
		btBodyApplyImpulse(sphere,TFormedX()*1000, TFormedY()*1000, TFormedZ()*1000)
		ent=0
	EndIf

	If KeyHit(57) Then
		If  ent<>-1
			If ent<>phSphereGetEntity() 
				ent=phSphereGetEntity()
				EntityParent (phSphereGetEntity(),pivotMainChair,1)
				;btBodySetPosition(sphere,EntityX(phSphereGetEntity(),0),EntityY(phSphereGetEntity(),0),EntityZ(phSphereGetEntity(),0))
				;btBodySetRotation(sphere,EntityPitch(phSphereGetEntity(),0),EntityYaw(phSphereGetEntity(),0),EntityRoll(phSphereGetEntity(),0))
				btBodySetPosition(sphere,EntityX(phSphereGetEntity(),1),EntityY(phSphereGetEntity(),1),EntityZ(phSphereGetEntity(),1))
				btBodySetRotation(sphere,EntityPitch(phSphereGetEntity(),1),EntityYaw(phSphereGetEntity(),1),EntityRoll(phSphereGetEntity(),1))
				btBodyCompoundAddBody(bodyCompound,sphere,1,0,1)
			EndIf
		EndIf
	EndIf

	btSimulation(.06,6)

	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)

	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	Text 10,70,"W,A,S,D - move camera"
	Text 10,90,"Left mouse - to throw a sphere"
	Text 10,110,"Space - add last sphere to body compound"

	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End

Function phCreateChairElement%(parent, dx#, dy#, dz#, posx#, posy#, posz#, rotx#=0, roty#=0, rotz#=0)
	
	body=btBodyCreateBox(dx,dy,dz,0)
	btBodySetPosition(body,posx,posy,posz)
	btBodySetRotation(body,rotx,roty,rotz)
	
	mesh=CreateCube()
	EntityColor mesh,100,120,70
	ScaleEntity (mesh,dx,dy,dz,1)
	PositionEntity (mesh,posx,posy,posz,1)
	RotateEntity (mesh,rotx,roty,rotz,1)
	EntityParent mesh,parent

	Return body
End Function

Type phEntity
	Field mesh
	Field body
	Field group
End Type

Function phCreateSphere%()
	
	phEnt.phEntity = New phEntity
	radius#=Rnd(4,6)
	phEnt\mesh=CreateSphere()
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,Rnd(5,10))
			
	If hide=1
		HideEntity phEnt\mesh
	EndIf

	EntityColor phEnt\mesh,Rand(100,200),Rand(100,200),Rand(100,200)
	
	btBodySetPosition(phEnt\body,Rnd(-50,150),Rnd(50,100),Rnd(-50,50))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	
	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)

	Return phEnt\body
End Function

Function phSphereGetEntity%()
	
	phEnt.phEntity = Last phEntity
	Return phEnt\mesh
	
End Function

Function HideShowMeshes()

	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh
			HideEntity meshPlane
			HideEntity meshMainChair
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
			ShowEntity meshPlane
			ShowEntity meshMainChair
		Next
		hide=0
	EndIf
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)
	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function