;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Compound decomposition"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/50)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)
btContactSetSolverIterations(6)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,40,-85
RotateEntity camera,5,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,25,0,20

Global meshPlane=CreatePlane()
Global tex=LoadTexture("..\media\cell.png")
Global tex_mesh=LoadTexture("..\media\cell.png")
ScaleTexture tex,10,10
EntityTexture meshPlane,tex,0,1 

btBodyCreatePlane(0,1,0)
shape=btShapeCreateCompound()
bodyCompound=btBodyCreateCompound(shape,600)
btBodySetAngularDamping(bodyCompound,0.8)
btBodySetLinearDamping(bodyCompound,0.27)
btBodySetFriction(bodyCompound,0.4)

pivot=CreatePivot()

index=0
lenRow=7
numRow=12
For i=1 To lenRow
	For j=1 To numRow
		body=phCreateCube(7,2,3,5)
		If (j Mod 2)=0 Then offset#=5 Else offset#=0

		btBodySetPosition(body,i*14-55+offset,4*j,0)
		btBodySetMargin(body,0.4)
		EntityParent phGetEntity(),pivot
		
		btBodyCompoundAddBody(bodyCompound,body,1,1,0)
		btBodyChildSetUserData(bodyCompound,index,phGetEntity())
		index=index+1
		
		;btBodySetActivationState(body,DISABLE_SIMULATION)
		;btBodyEnableResponse(body,0)
		btWorldRemoveBody(body)
	Next
Next

btBodySetPosition(bodyCompound,0,25,0)
btBodyAttachEntity(bodyCompound,pivot)

Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
HidePointer 

Repeat	
	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	If KeyHit(35) Then HideShowMeshes()
	
	msx#=MouseXSpeed()
	msy#=MouseYSpeed()
	Camera_Look(camera)

	If MouseHit(1)
		cube=phCreateCube(2,2,2,10,2)
		TFormVector (0,0,1,camera,0)
		btBodySetPosition cube,EntityX(camera)+TFormedX()*10,EntityY(camera)+TFormedY()*10,EntityZ(camera)+TFormedZ()*10
		btBodyApplyImpulse(cube,TFormedX()*1500, TFormedY()*1500, TFormedZ()*1500)
	EndIf

	If MouseHit(2)
		sphere=phCreateSphere()
		TFormVector (0,0,1,camera,0)
		btBodySetPosition sphere,EntityX(camera)+TFormedX()*10,EntityY(camera)+TFormedY()*10,EntityZ(camera)+TFormedZ()*10
		btBodyApplyImpulse(sphere,TFormedX()*1200, TFormedY()*1200, TFormedZ()*1200)
	EndIf

	body_shot=phGetBody()
	If body_shot<>0 Then
		btContactTestAddBody(body_shot,0)
		btContactManifoldTest(True)
		btContactGetNum(body_shot)
	
		For i=0 To btContactGetNum(body_shot)-1
			If btBodyGetLinearVelocity(body_shot)>10 Then
				indexChild=btContactGetIndexA(body_shot,i)
				If indexChild<>-1 Then
					entity=btBodyChildGetUserData(bodyCompound,indexChild)
					If entity<>-1 Then
						EntityParent entity,0
				
						For phEnt.phEntity = Each phEntity
							If phEnt\mesh=entity
								x=btBodyChildGetX(bodyCompound,indexChild,1)
								y=btBodyChildGetY(bodyCompound,indexChild,1)
								z=btBodyChildGetZ(bodyCompound,indexChild,1)	
								btBodySetPosition(phEnt\body,x,y,z)
	
								pitch#=btBodyChildGetPitch(bodyCompound,indexChild,1)
								yaw#=btBodyChildGetYaw(bodyCompound,indexChild,1)
								roll#=btBodyChildGetRoll(bodyCompound,indexChild,1)
		
								btBodySetRotation(phEnt\body,pitch,yaw,roll)
								btWorldAddBody(phEnt\body)
								btBodySetActivationState(phEnt\body,ACTIVE_TAG)
								;btBodyEnableResponse(phEnt\body,1)
								Exit
							EndIf
						Next
						btBodyRemoveChild(bodyCompound,indexChild,0)
					EndIf
					Exit
				EndIf
			EndIf
		Next

		
	EndIf

	btSimulation(.05,5)

	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)
	
	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	Text 10,70,"Left mouse button - to throw a cube for destroying"
	Text 10,90,"Right mouse button - to throw a normal sphere"
	Text 10,110,"Num children: "+btBodyGetNumChildren(bodyCompound)

	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
	Field group
	Field ph_type
End Type

Function phCreateSphere%()
	
	phEnt.phEntity = New phEntity
	radius#=Rnd(4,6)
	phEnt\mesh=CreateSphere()
	EntityPickMode phEnt\mesh,2
	EntityShininess phEnt\mesh,0.1
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,Rnd(12,17))
			
	If hide=1
		EntityAlpha phEnt\mesh,0
	EndIf

	EntityColor phEnt\mesh,Rnd(110,180),Rnd(110,180),Rnd(110,180)
	
	btBodySetPosition(phEnt\body,Rnd(-50,150),Rnd(50,100),Rnd(-50,50))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)

	phEnt\ph_type=1
	
	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)

	Return phEnt\body
End Function

Function phCreateCube(dx#, dy#, dz#, mass#=10, ph_type=0)
	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateCube()

	If hide=1
		EntityAlpha phEnt\mesh,0
	EndIf
	
	EntityPickMode phEnt\mesh,2
	EntityShininess phEnt\mesh,0.1
	ScaleEntity phEnt\mesh,dx,dy,dz
	phEnt\body=btBodyCreateBox(dx,dy,dz,mass)
	EntityColor phEnt\mesh,Rnd(110,180),Rnd(110,180),Rnd(110,180)
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.2)
	btBodySetLinearDamping(phEnt\body,0.2)
	
	btBodySetAngularSleepingThreshold(phEnt\body,0.08)
	btBodySetLinearSleepingThreshold(phEnt\body,0.06)

	phEnt\ph_type=ph_type

	Return phEnt\body
End Function

Function phGetEntity%()
	
	phEnt.phEntity = Last phEntity
	Return phEnt\mesh
	
End Function

Function phGetBody%()
	
	phEnt.phEntity = Last phEntity
	If phEnt\ph_type=2
		Return phEnt\body
	Else
		Return 0
	EndIf
	
End Function

Function HideShowMeshes()

	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			EntityAlpha phEnt\mesh,0
			HideEntity meshPlane
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			EntityAlpha phEnt\mesh,1
			ShowEntity meshPlane
		Next
		hide=0
	EndIf
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)

	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function

Function ActiveCamera(cam,pivot_car,ent_car,cspeedX#,cspeedY#,cspeedZ#,dist#,hite#,xrot#,Modecamera)

	If KeyDown (30) Then 
		TurnEntity pivot_car,0,-1,0
	Else	
		If KeyDown (32) Then TurnEntity pivot_car,0,1,0
 	EndIf
	
	Select Modecamera	  	    
		Case 0
		    EntityParent cam,0
			PointEntity cam,pivot_car
			
			TFormPoint 0,hite,dist,pivot_car,0
			cx#=(TFormedX()-EntityX(cam))*(cspeedX#)
			cy#=(TFormedY()-EntityY(cam))*(cspeedY#)
			cz#=(TFormedZ()-EntityZ(cam))*(cspeedZ#)
			TranslateEntity cam,cx,cy,cz
			flagPosCamera=0
		Case 2		 
		    RotateEntity cam,0,0,0
			PositionEntity cam,0,35,15
			EntityParent cam,ent_car	
			flagPosCamera=0						
		Case 3
		    EntityParent cam,ent_car
			RotateEntity cam,0,0,0
			PositionEntity cam,-10,27,-4
			flagPosCamera=0	
		Case 4
			RotateEntity cam,0,0,0
			PositionEntity cam,-27,15,-2
			flagPosCamera=0	
		Case 1
			EntityParent cam,0
			If flagPosCamera=0
				PositionEntity cam,400,170,-260
				flagPosCamera=1
			EndIf
			PointEntity cam,ent_car	   	    	
	End Select 
	
End Function
