;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Debug draw single"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(DBG_WF)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverIterations(8)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,80,-80
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,25,0,20

Global meshPlane=CreatePlane()
Global tex=LoadTexture("..\media\cell.png")
Global tex_mesh=LoadTexture("..\media\cell.png")
ScaleTexture tex,20,20
EntityTexture meshPlane,tex,0,1 

bodyPlane=btBodyCreatePlane(0,1,0)
btBodySetRestitution(bodyPlane,0.6)

body=CreatePrimitive(0)
btBodySetPosition(body,-50,50,0)

body=CreatePrimitive(1)
btBodySetPosition(body,-25,50,0)

body=CreatePrimitive(2)
btBodySetPosition(body,0,50,0)

body=CreatePrimitive(3)
btBodySetPosition(body,25,50,0)

body=CreatePrimitive(4)
btBodySetPosition(body,50,50,0)

Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

btRay=btCreateRay()
btRaySetMaxDistance(btRay,500)

btMeshLinesOrder=-1000

Repeat

	mouse_left_hit=MouseHit(1) 
	mouse_right_hit=MouseHit(2) 

	btRaySetPosition(btRay,EntityX(camera,1),EntityY(camera,1),EntityZ(camera,1))
	
	CameraPick(camera,MouseX(),MouseY())
	If PickedEntity()<>0
		entity=PickedEntity()
		If mouse_left_hit=1 Or mouse_right_hit=1
			btRaySetDirection(btRay,PickedX#()-EntityX(camera,1), PickedY#()-EntityY(camera,1), PickedZ#()-EntityZ(camera,1))
			btRayGetPickInfo(btRay,0)
			pickBody=btPickInfoBody()

			If(pickBody<>0)
				If mouse_right_hit=1
					btDebugDrawBodySetColor(pickBody,Rnd(0,255),Rnd(0,255),Rnd(0,255))
				Else			
					If btDebugDrawFindObject(pickBody)
						btDebugDrawRemoveObject(pickBody)
					Else
						btDebugDrawAddBody(pickBody)
					EndIf
					
				EndIf
			EndIf
		EndIf
	EndIf
	
	btSimulation(.1,2)

	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera,1)
	
	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"Left mouse click on entity - draw/not draw"
	Text 10,50,"Right mouse click on entity - select new color"

	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End



Type phEntity
	Field mesh
	Field body
	Field group
End Type


Function CreateCapsule(height#, radius#)

capsule= CreateCylinder()
ScaleMesh capsule, radius, height/2, radius
PositionMesh capsule,0,0,0

sphere1 = CreateSphere() 
ScaleMesh sphere1,radius,radius,radius
PositionMesh sphere1,0, height/2,0
AddMesh sphere1,capsule

sphere2 = CreateSphere() 
ScaleMesh sphere2,radius,radius,radius
PositionMesh sphere2,0, -height/2,0
AddMesh sphere2,capsule

FreeEntity sphere1
FreeEntity sphere2
;free capsule

Return capsule

End Function

Function CreatePrimitive(pType)

	phEnt.phEntity = New phEntity
	Select pType
	Case 0
		phEnt\mesh=CreateCapsule(12,7)
		phEnt\body=btBodyCreateCapsule(7,12,0)
	Case 1
		dx#=Rnd(5,8)
		dy#=Rnd(5,8)
		dz#=Rnd(5,8)
		phEnt\mesh=CreateCube()
		ScaleEntity phEnt\mesh,dx,dy,dz
		phEnt\body=btBodyCreateBox(dx,dy,dz,0)
	Case 2
		radius#=Rnd(7,12)
		phEnt\mesh=CreateSphere()
		ScaleEntity phEnt\mesh,radius,radius,radius
		phEnt\body=btBodyCreateSphere(radius,0)
	Case 3
		radius#=10
		height#=10
		phEnt\mesh=CreateCone()
		ScaleEntity phEnt\mesh,radius,height,radius
		phEnt\body=btBodyCreateCone(radius,height,0)
	Case 4
		radius#=6
		height#=10
		phEnt\mesh=CreateCylinder()
		ScaleEntity phEnt\mesh,radius,height,radius
		phEnt\body=btBodyCreateCylinder(radius,height,0)
	End Select 
	If hide=1
		HideEntity phEnt\mesh
	EndIf

	EntityPickMode phEnt\mesh,2
	EntityColor phEnt\mesh,Rand(100,200),Rand(100,200),Rand(100,200)
	;btBodySetPosition(phEnt\body,Rnd(-150,150),Rnd(50,100),Rnd(-150,150))
	;btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)

	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)

	Return phEnt\body
End Function

Function phGetBody%()
	
	phEnt.phEntity = Last phEntity
	Return phEnt\body
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)
	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function