;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Scaling"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(DBG_JOINTS+DBG_JOINTLIMITS)
btDebugDrawEnableNodeTree(0)
btWorldSetFixedTimeStep(1.0/60)
btContactSetSolverIterations(5)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)
btSoftWorldSetGravity(0,-10,0)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,50,-100
RotateEntity camera,5,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,20,0,20

Global meshPlane=CreatePlane()
Global tex=LoadTexture("..\media\cell.png")
Global tex_mesh=LoadTexture("..\media\cell.png")
ScaleTexture tex,10,10
EntityTexture meshPlane,tex,0,1 

btBodyCreateBox(500,1,500,0)


meshTorus=LoadMesh("..\media\torus.b3d")
RotateMesh meshTorus,90,0,0
ScaleMesh meshTorus,0.3,0.3,0.3
EntityPickMode meshTorus,2

softBody=btSoftBodyCreateFromMesh(meshTorus,10,2)
btBodySetPosition(softBody,40,40,0)

btSoftBodySetFlagCollisions(softBody,SC_CLRS)
btSoftBodyGenerateClusters(softBody,32)
btSoftBodySetAngularStiffness(softBody,1)
btSoftBodySetLinearStiffness(softBody,0.4)
btSoftBodySetVolumeStiffness(softBody,1)
btSoftBodySetPressure(softBody,1000)
btSoftBodySetFlagCollisions(softBody,2)
btSoftBodySetDrag(softBody,100)

meshChair=LoadMesh("..\media\stool.b3d")
EntityPickMode meshChair,2
ScaleMesh meshChair,0.7,0.7,0.7
EntityColor meshChair,100,140,200
bodyConcave=btBodyCreateTrimesh(meshChair,10,TRIMESH_CONCAVE)
btBodySetMass(bodyConcave,0)
btBodySetPosition(bodyConcave,0,30,0)
btBodySetRotation(bodyConcave,0,50,0)
btBodyAttachEntity(bodyConcave,meshChair)

meshCube=CreateCube()
EntityPickMode meshCube,2
ScaleMesh meshCube,7,7,7
EntityColor meshCube,200,140,100
bodyConvex=btBodyCreateConvexHull(meshCube,7,1)
btBodySetMass(bodyConvex,0)
btBodySetPosition(bodyConvex,-30,30,0)
btBodySetRotation(bodyConvex,0,50,0)
btBodyAttachEntity(bodyConvex,meshCube)

btRay=btCreateRay()

Global pickBody=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

Repeat	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	btSoftBodyUpdateEntity(softBody,meshTorus)
	btRaySetPosition(btRay,EntityX(camera,1),EntityY(camera,1),EntityZ(camera,1))
	mouse_hit=MouseHit(1)

	CameraPick(camera,MouseX(),MouseY())
	If PickedEntity()<>0
		btRaySetDirection(btRay,PickedX#()-EntityX(camera,1), PickedY#()-EntityY(camera,1), PickedZ#()-EntityZ(camera,1))
		If mouse_hit=1 Then
			btRayGetPickInfo(btRay,0)
			pickBody=btPickInfoBody()
			bx#=btBodyGetX(pickBody)
			by#=btBodyGetY(pickBody)
			bz#=btBodyGetZ(pickBody)
			btBodySetScale(pickBody,1.5,1.5,1.5)

			If (btBodyIsSoftBody(pickBody))
				btBodySetPosition(pickBody,bx,by,bz)
				btSoftBodySetDamping(pickBody,0.004)
				btSoftBodySetMass(pickBody,1,145)
				btSoftBodySetDrag(pickBody,0)
			Else
				ScaleEntity PickedEntity(),1.5,1.5,1.5
				btBodySetMass(pickBody,20)
			EndIf
		EndIf
	EndIf
	
	btSimulation(.09,6)
	
	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera,0)

	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"Click on entity for scaling"
	
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End
