;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Character controller"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/60)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,40,-70
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,25,0,20

Global meshPlane=CreatePlane()
Global tex=LoadTexture("..\media\cell.png")
Global tex_mesh=LoadTexture("..\media\cell.png")
ScaleTexture tex,20,20
EntityTexture meshPlane,tex,0,1 

btBodyCreatePlane(0,1,0)

characterController=btCreateCharacterController(7,22)
btCControllerSetPosition(characterController,0,20,0)
btCControllerSetMaxJumpHeight(characterController,2)
btCControllerSetJumpSpeed(characterController,20)
btCControllerSetFallSpeed(characterController,20)
btCControllerSetMaxSlope(characterController,80)

bodyCharacter=btCControllerGetBody(characterController)
meshCharacter=CreateCapsule(7,22)
btBodyAttachEntity(bodyCharacter,meshCharacter)
EntityParent camera,meshCharacter

btCControllerSetGravity(characterController,9)

For i=1 To 20
	bodyNext=phCreateCube(Rnd(6,7),Rnd(6,7),Rnd(6,7),Rnd(1,2))
	btBodySetPosition(bodyNext,Rnd(-120,120),Rnd(10,70),Rnd(-120,120))
Next

For i=0 To 20
	bodyNext=phCreateCube(5,2,40,0)
	btBodySetPosition(bodyNext,-120-(i*9),2+(i*3),0)
Next


entity=0
Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

Repeat	
	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	msx#=MouseXSpeed()
	msy#=MouseYSpeed()
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

	btBodySetRotation(bodyCharacter, btBodyGetPitch(bodyCharacter), btBodyGetYaw(bodyCharacter)-(msx*0.2), btBodyGetRoll(bodyCharacter))

	If KeyDown(17)
		speedz#=0.2
	Else
		If KeyDown(31)
			speedz#=-0.2
		Else
			speedz#=0
		EndIf
	EndIf

	If KeyDown(30)
		speedx#=-0.2
	Else
		If KeyDown(32)
			speedx#=0.2
		Else
			speedx#=0
		EndIf
	EndIf

	If KeyDown(57)
		btCControllerJump(characterController)
	EndIf

	btCControllerSetWalkDirection(characterController,speedx,speedy,speedz,0)
	
	btSimulation(.1,6)
	
	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)
	
	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"W,A,S,D - movement"
	Text 10,70,"Space - jump"
	
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End



Type phEntity
	Field mesh
	Field body
	Field group
End Type

Function phCreateSphere%(radius, mass)
	
	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateSphere()
	EntityPickMode phEnt\mesh,2
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,mass)
			
	If hide=1
		EntityAlpha phEnt\mesh,0
	EndIf

	EntityColor phEnt\mesh,Rnd(150,220),Rnd(150,220),Rnd(150,220)
	
	btBodySetPosition(phEnt\body,Rnd(-50,150),Rnd(50,100),Rnd(-50,50))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	
	btBodySetAngularSleepingThreshold(phEnt\body,.01)
	btBodySetLinearSleepingThreshold(phEnt\body,.01)
	
	btBodySetFriction(phEnt\body,0.8)

	Return phEnt\body
End Function

Function phCreateCube(dx#, dy#, dz#, mass#=10)
	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateCube()
	EntityPickMode phEnt\mesh,2
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,dx,dy,dz
	phEnt\body=btBodyCreateBox(dx,dy,dz,mass)
	EntityColor phEnt\mesh,Rnd(150,220),Rnd(150,220),Rnd(150,220)
	;ScaleTexture tex_mesh,1,1
	;EntityTexture phEnt\mesh,tex_mesh,0,1 
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.02)
	btBodySetLinearDamping(phEnt\body,0.2)
	
	btBodySetAngularSleepingThreshold(phEnt\body,0.08)
	btBodySetLinearSleepingThreshold(phEnt\body,0.06)

	Return phEnt\body
End Function

Function CreateCapsule(radius#,height#)

	capsule= CreateCylinder()
	ScaleMesh capsule, radius, height/2, radius
	PositionMesh capsule,0,0,0
	EntityPickMode capsule,2
	EntityColor capsule,150,150,90
	
	sphere1 = CreateSphere() 
	ScaleMesh sphere1,radius,radius,radius
	PositionMesh sphere1,0, height/2,0
	AddMesh sphere1,capsule
	
	sphere2 = CreateSphere() 
	ScaleMesh sphere2,radius,radius,radius
	PositionMesh sphere2,0, -height/2,0
	AddMesh sphere2,capsule
	
	FreeEntity sphere1
	FreeEntity sphere2
	;HideEntity capsule
	;EntityAlpha capsule,0.1
	
	Return capsule
End Function

Function phGetEntity%()
	
	phEnt.phEntity = Last phEntity
	Return phEnt\mesh
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)
	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function