;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Mouse drag"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(DBG_JOINTS+DBG_JOINTLIMITS)
btDebugDrawEnableNodeTree(0)
btDebugDrawSetFlagSoftBody(1+16)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverIterations(5)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,50,-100
RotateEntity camera,5,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,20,0,20

Global meshPlane=CreatePlane()
Global tex=LoadTexture("..\media\cell.png")
Global tex_mesh=LoadTexture("..\media\cell.png")
ScaleTexture tex,10,10
EntityTexture meshPlane,tex,0,1 

btBodyCreateBox(500,1,500,0)

numRow=8
numPass=0
tempNumRow=numRow
For i=1 To numRow
	numPass=numPass+tempNumRow
	tempNumRow=tempNumRow-1
Next

posx#=70
posy#=4.5
memIndex=0
For i=0 To numPass-1
	If ( (i-memIndex)=numRow )
		numRow=numRow-1
		memIndex=i
		posy=posy+8.2
		posx=posx-4
	EndIf

	body=phCreateCube(4,4,4,Rnd(10,20))
	btBodySetPosition(body,((i-memIndex)*9.2)-posx,posy,0)
	btBodySetRotation(body,0,60,0)
Next

Global meshTorus=LoadMesh("..\media\torus.b3d")
RotateMesh meshTorus,90,0,0
ScaleMesh meshTorus,0.5,0.5,0.4
EntityPickMode meshTorus,2

softBody=btSoftBodyCreateFromMesh(meshTorus,10,2)
btSoftBodyGenerateClusters(softBody,32)
btBodySetRotation(softBody,-5,0,0)
btBodySetPosition(softBody,40,40,20)
btSoftBodySetAngularStiffness(softBody,0.01)
btSoftBodySetLinearStiffness(softBody,0.3)
btSoftBodySetVolumeStiffness(softBody,0.01)
btSoftBodySetPressure(softBody,100)
btSoftBodySetFlagCollisions(softBody,2)
btSoftBodySetDamping(softBody,0.006)
btSoftBodySetPose(softBody,False,False)
btSoftBodySetPoseMatching(softBody,0.5)

controlBody=btBodyCreateBox(1,1,1,0)
btBodyEnableResponse(controlBody,0)
btBodySetActivationState(controlBody,DISABLE_DEACTIVATION)

btRay=btCreateRay()

Const use_point2pont=1
Global control_joint=0
Global control_softbody=0
Global pickBody=0
Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

Repeat	
	If KeyHit(2) btDebugDrawSetMode(DBG_JOINTS+DBG_JOINTLIMITS)
	If KeyHit(3) btDebugDrawSetMode(1+DBG_JOINTLIMITS+DBG_JOINTS)
	If KeyHit(4) btDebugDrawSetMode(2+DBG_JOINTLIMITS+DBG_JOINTS)
	If KeyHit(5) btDebugDrawSetMode(1+2+DBG_JOINTLIMITS+DBG_JOINTS)

	If KeyHit(35) Then HideShowMeshes()

	msx#=MouseXSpeed()
	msy#=MouseYSpeed()
	
	btSoftBodyUpdateEntity(softBody,meshTorus)
	btRaySetPosition(btRay,EntityX(camera,1),EntityY(camera,1),EntityZ(camera,1))
	mouse_hit=MouseHit(1)

	CameraPick(camera,MouseX(),MouseY())
	If PickedEntity()<>0 And control_joint=0
		If mouse_hit Then
			btRaySetDirection(btRay,PickedX#()-EntityX(camera,1), PickedY#()-EntityY(camera,1), PickedZ#()-EntityZ(camera,1))
			If btRayGetPickInfo(btRay,1)
				pickBody=btPickInfoBody()
				If pickBody<>0
					cx#=btPickInfoX()
					cy#=btPickInfoY()
					cz#=btPickInfoZ()
					btBodySetPosition(controlBody,cx,cy,cz)
					btBodySetActivationState(pickBody,DISABLE_DEACTIVATION)
					If btBodyIsSoftBody(pickBody)
						control_joint=btSoftBodyAppendAnchor(pickBody,btPickInfoNode(),controlBody,1)
						control_softbody=1
					Else
						btBodySetSleep(pickBody,1)
						If use_point2pont=1
							control_joint=btJointCreatePoint2Point(pickBody,controlBody, cx,cy,cz,1, cx,cy,cz,1, 0)
						Else
							control_joint=btJointCreate6DofGeneric(pickBody,controlBody, cx,cy,cz,1, cx,cy,cz,1, 0)
							
							btJointSetParam(control_joint,BT_JOINT_STOP_CFM,0.8,0)
							btJointSetParam(control_joint,BT_JOINT_STOP_CFM,0.8,1)
							btJointSetParam(control_joint,BT_JOINT_STOP_CFM,0.8,2)
							btJointSetParam(control_joint,BT_JOINT_STOP_CFM,0.8,3)
							btJointSetParam(control_joint,BT_JOINT_STOP_CFM,0.8,4)
							btJointSetParam(control_joint,BT_JOINT_STOP_CFM,0.8,5)
		
							btJointSetParam(control_joint,BT_JOINT_STOP_ERP,0.1,0)
							btJointSetParam(control_joint,BT_JOINT_STOP_ERP,0.1,1)
							btJointSetParam(control_joint,BT_JOINT_STOP_ERP,0.1,2)
							btJointSetParam(control_joint,BT_JOINT_STOP_ERP,0.1,3)
							btJointSetParam(control_joint,BT_JOINT_STOP_ERP,0.1,4)
							btJointSetParam(control_joint,BT_JOINT_STOP_ERP,0.1,5)
						EndIf
						
						control_softbody=0
					EndIf
				EndIf
			EndIf
		EndIf
	EndIf

	If MouseDown(1) Then
		If pickBody<>0
			cx# = cx + msx/4
			cy# = cy - msy/4
			btBodySetPosition(controlBody,cx,cy,cz)
			CameraProject camera,btBodyGetX(controlBody),btBodyGetY(controlBody),btBodyGetZ(controlBody)
			MoveMouse ProjectedX(),ProjectedY()
		EndIf
	Else
		If control_joint<>0 Then
			If control_softbody=1
				btSoftBodyDeleteAnchor(control_joint)
			Else
				btDeleteJoint(control_joint)
			EndIf
			control_joint=0
			btBodySetActivationState(pickBody,ACTIVE_TAG)
			pickBody=0
		EndIf
	EndIf

	btSimulation(.1,5)


	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera,0)

	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
	Field group
End Type

Function phCreateSphere%()
	
	phEnt.phEntity = New phEntity
	radius#=Rnd(4,6)
	phEnt\mesh=CreateSphere()
	EntityPickMode phEnt\mesh,2
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,Rnd(5,10))
			
	If hide=1
		EntityAlpha phEnt\mesh,0
	EndIf

	EntityColor phEnt\mesh,red,green,blue
	
	btBodySetPosition(phEnt\body,Rnd(-50,150),Rnd(50,100),Rnd(-50,50))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	
	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)

	Return phEnt\body
End Function

Function phCreateCube(dx#, dy#, dz#, mass#=10)
	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateCube()
	EntityPickMode phEnt\mesh,2
	EntityShininess phEnt\mesh,0.1
	ScaleEntity phEnt\mesh,dx,dy,dz
	phEnt\body=btBodyCreateBox(dx,dy,dz,mass)
	EntityColor phEnt\mesh,Rnd(100,140),Rnd(100,140),Rnd(100,140)

	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.2)
	btBodySetLinearDamping(phEnt\body,0.4)
	btBodySetAngularDamping(phEnt\body,0.5)
	
	btBodySetAngularSleepingThreshold(phEnt\body,0.5)
	btBodySetLinearSleepingThreshold(phEnt\body,0.7)

	Return phEnt\body
End Function

Function phSphereGetEntity%()
	
	phEnt.phEntity = Last phEntity
	Return phEnt\mesh
	
End Function

Function HideShowMeshes()

	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			EntityAlpha phEnt\mesh,0
			HideEntity meshPlane
			EntityAlpha meshTorus,0
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			EntityAlpha phEnt\mesh,1
			ShowEntity meshPlane
			EntityAlpha meshTorus,1
		Next
		hide=0
	EndIf
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)

	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function
