;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Explosion"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverIterations(8)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,80,-150
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,25,0,20

Global meshPlane=CreatePlane()
Global tex=LoadTexture("..\media\cell.png")
Global tex_mesh=LoadTexture("..\media\cell.png")
ScaleTexture tex,20,20
EntityTexture meshPlane,tex,0,1 
EntityAlpha meshPlane,0.7
CreateMirror()

bodyPlane=btBodyCreatePlane(0,1,0)
btBodySetRestitution(bodyPlane,0.6)

For i=1 To 6
	For j=1 To 7
		For k=1 To 6
			body=phCreateCube(5,5,5,Rnd(10,15))
			btBodySetPosition(body,-35+i*11,j*10.2-4,-40+k*11)
		Next
	Next
Next

CreateListener(camera,0.05)
soundExplosion=Load3DSound("..\media\Explosion.wav")

Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2

Repeat	
	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	msx#=MouseXSpeed()
	msy#=MouseYSpeed()
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	Camera_Look(camera)
	
	If MouseHit(1)
		sphere=phCreateBomb(4,10)
		EntityColor phBombGetEntity(),10,10,10
		TFormVector (0,0,1,camera,0)
		btBodySetPosition sphere,EntityX(camera)+TFormedX()*10,EntityY(camera)+TFormedY()*10,EntityZ(camera)+TFormedZ()*10
		btBodyApplyImpulse(sphere,TFormedX()*1000, TFormedY()*1000, TFormedZ()*1000)
	EndIf

	For bomb.phBomb = Each phBomb 
		bomb\timerExplosion=bomb\timerExplosion+1
		
		If bomb\state=0
			btContactTestAddBody(bomb\body,0)
			If btContactManifoldTest(True)>0 Then
				For i=0 To btContactGetNum(bomb\body)-1
					bodyType=btBodyGetType(btContactGetBody(bomb\body,i))
					If bodyType<>1 And bodyType<>3 Then
						bomb\state=1
						bomb\timerExplosion=0
						Exit
					EndIf
				Next
			EndIf
		EndIf
		
		If bomb\timerExplosion>500 Then
			If bomb\state=0
				bomb\state=1
				bomb\timerExplosion=0
			EndIf
		EndIf
		
		If bomb\state=1
			For phEnt.phEntity = Each phEntity
				bx#=btBodyGetX(bomb\body)
				by#=btBodyGetY(bomb\body)
				bz#=btBodyGetZ(bomb\body)
				forcex#=((-btBodyGetDirectionToPointX(phEnt\body,bx,by,bz)*1000+1)/(btBodyGetDistance(phEnt\body,bomb\body)*0.01))
				forcey#=((-btBodyGetDirectionToPointY(phEnt\body,bx,by,bz)*1000+1)/(btBodyGetDistance(phEnt\body,bomb\body)*0.01))
				forcez#=((-btBodyGetDirectionToPointZ(phEnt\body,bx,by,bz)*1000+1)/(btBodyGetDistance(phEnt\body,bomb\body)*0.01))
				btBodyApplyForce(phEnt\body,forcex,forcey,forcez)
			Next
			bomb\state=2
			EntityColor bomb\mesh,255,0,0
				
			EmitSound(soundExplosion,bomb\mesh)			
			btDeleteBody(bomb\body)
		Else
			If bomb\state=2
				If bomb\timerExplosion<50 Then
					bomb\alpha=bomb\alpha-0.03
					EntityAlpha bomb\mesh,bomb\alpha
					
					bomb\scale=bomb\scale+1.4
					ScaleEntity bomb\mesh,bomb\scale,bomb\scale,bomb\scale
				Else
					FreeEntity bomb\mesh
					Delete bomb
				EndIf
			EndIf
		EndIf
	Next

	btSimulation(.1,2)
	
	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)
	
	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"Left mouse - to throw a bomb"
	
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End



Type phEntity
	Field mesh
	Field body
	Field group
End Type

Type phBomb
	Field mesh
	Field body
	Field timerExplosion
	Field state
	Field alpha#
	Field scale#
End Type


Function phCreateBomb%(radius, mass)
	
	bomb.phBomb = New phBomb
	bomb\mesh=CreateSphere()
	EntityPickMode bomb\mesh,2
	EntityShininess bomb\mesh,0.5
	ScaleEntity bomb\mesh,radius,radius,radius
	bomb\body=btBodyCreateSphere(radius,mass)
			
	If hide=1
		EntityAlpha bomb\mesh,0
	EndIf

	EntityColor bomb\mesh,Rnd(150,220),Rnd(150,220),Rnd(150,220)
	
	btBodySetPosition(bomb\body,Rnd(-50,150),Rnd(50,100),Rnd(-50,50))
	btBodySetRotation(bomb\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(bomb\body,bomb\mesh)
	btBodySetRestitution(bomb\body,0.9)
	
	btBodySetAngularSleepingThreshold(bomb\body,.01)
	btBodySetLinearSleepingThreshold(bomb\body,.01)
	
	btBodySetLinearDamping(bomb\body,0.2)
	
	btBodySetFriction(bomb\body,0.02)
	bomb\timerExplosion=0
	bomb\state=False
	bomb\alpha=1
	bomb\scale=1
	
	Return bomb\body
End Function

Function phCreateCube(dx#, dy#, dz#, mass#=10)

	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateCube()
	EntityPickMode phEnt\mesh,2
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,dx,dy,dz
	phEnt\body=btBodyCreateBox(dx,dy,dz,mass)
	EntityColor phEnt\mesh,Rnd(150,220),Rnd(150,220),Rnd(150,220)
	;ScaleTexture tex_mesh,1,1
	;EntityTexture phEnt\mesh,tex_mesh,0,1 
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.1)
	btBodySetLinearDamping(phEnt\body,0.2)
	btBodySetRestitution(phEnt\body,0.0)
	
	btBodySetAngularSleepingThreshold(phEnt\body,0.8)
	btBodySetLinearSleepingThreshold(phEnt\body,0.5)

	Return phEnt\body
	
End Function

Function phBombGetEntity%()
	
	bomb.phBomb = Last phBomb
	Return bomb\mesh
	
End Function

Function phGetBody%()
	
	phEnt.phEntity = Last phEntity
	Return phEnt\body
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)
	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function