;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Pendulums on soft bodies"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/60)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,90,-85
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,20,0,20

Global red=120,green=120,blue=250
Global meshPlane=CreatePlane()
Global tex=LoadTexture("..\media\cell.png")
Global tex_mesh=LoadTexture("..\media\cell.png")
ScaleTexture tex,10,10
EntityTexture meshPlane,tex,0,1 

begin_posx#=-55
begin_posy#=80
begin_length#=6
For i=1 To 12
	body=phCreatePendulum(begin_posx,begin_posy,0,begin_length)
	begin_posx=begin_posx+10
	begin_length=begin_length-0.3
	begin_posy=begin_posy-2.02
	btBodyApplyForce(body,0,0,1500)
Next

Type tPendulums
	Field mesh_rope
	Field mesh_sphere
	Field rope
	Field array_nodes
	Field sphere
	Field x#,y#,z#
End Type

Function phCreatePendulum(x#,y#,z#,length#=5,radius#=4.5,mass#=10)
	meshRope=LoadMesh("..\media\rope.b3d")
	ScaleMesh meshRope,0.2,1,0.1*length
	RotateMesh meshRope,-90,0,0
	EntityColor meshRope,100,100,100
	EntityFX meshRope,16+1	
	MeshCullBox meshRope,-50,-50,-50,100,100,100
		
	bodyRope=btSoftBodyCreateFromMesh(meshRope,10,2)
	btBodySetPosition(bodyRope,x,y,z)
	btBodyAttachEntity (bodyRope,meshRope)
	btSoftBodySetFlagCollisions(bodyRope,2)
	btSoftBodySetLinearStiffness(bodyRope,0.02)
	btBodySetMargin(bodyRope,0.8)
	
	btSoftBodySetMass(bodyRope,0,16) 
	btSoftBodySetMass(bodyRope,0,17)
	btSoftBodyGenerateClusters(bodyRope,1)

	btSoftBodySetClusterRigidHardness(bodyRope,0.01) 

	meshSphere=CreateSphere()
	EntityPickMode meshSphere,2
	EntityColor meshSphere,Rnd(150,120),Rnd(150,120),Rnd(150,120)
	ScaleEntity meshSphere,radius,radius,radius
	bodySphere=btBodyCreateSphere(radius,mass)
	btBodyAttachEntity (bodySphere,meshSphere)
	btBodySetPosition(bodySphere,x,btSoftBodyNodeGetY(bodyRope,1),z)
	EntityShininess (meshSphere,0.5)

	btBodySetLinearDamping(bodySphere,0.04)

	btSoftBodyAppendAnchor(bodyRope,1,bodySphere,1)
	btSoftBodyAppendAnchor(bodyRope,3,bodySphere,1)

	btSoftBodySetAnchorHardness(bodyRope,0.5)
	
	p.tPendulums = New tPendulums
	p\rope=bodyRope
	p\sphere=bodySphere
	p\array_nodes=btSoftBodyCreateArrayNodes(p\rope)
	p\x=btBodyGetX(bodySphere)
	p\y=btBodyGetY(bodySphere)
	p\z=btBodyGetZ(bodySphere)
	
	p\mesh_rope=meshRope
	p\mesh_sphere=meshSphere
	
	Return bodySphere
End Function

Function phResetPendulums()

	For p.tPendulums = Each tPendulums
		btSoftBodySetVelocity(p\rope,0,0,0)
		btSoftBodySetArrayNodes(p\array_nodes,1)
		btBodySetPosition(p\sphere,p\x,p\y,p\z)
		btBodySetLinearVelocity(p\sphere,0,0,0)
		btBodySetAngularVelocity(p\sphere,0,0,0)
	Next
	
End Function

Function phPendulumsApplyForce(mode)
	
	Select mode
	Case 0
		For p.tPendulums = Each tPendulums	
			btBodyApplyForce(p\sphere,0,0,1500)
		Next
	Case 1
		force=700
		For p.tPendulums = Each tPendulums	
			btBodyApplyForce(p\sphere,0,0,force)
			force=force+150
		Next
	Case 2
		force=2000
		For p.tPendulums = Each tPendulums	
			btBodyApplyForce(p\sphere,0,0,force)
			force=force-100
		Next
	Case 3
		force=1500
		index=1
		For p.tPendulums = Each tPendulums	
			If (index Mod 2)=0 Then
				btBodyApplyForce(p\sphere,0,0,force)
			Else
				btBodyApplyForce(p\sphere,0,0,-force)
			EndIf
			index=index+1		
		Next
	End Select
	
End Function

Function phHidePendulums()

	For p.tPendulums = Each tPendulums	
		EntityAlpha p\mesh_rope,0
		EntityAlpha p\mesh_sphere,0
	Next
	
End Function

Function phShowPendulums()

	For p.tPendulums = Each tPendulums	
		EntityAlpha p\mesh_rope,1
		EntityAlpha p\mesh_sphere,1
	Next
	
End Function


btRay=btCreateRay()
btRayEnablePickSoftBody(btRay,1)

controlBody=btBodyCreateBox(1,1,1,0)
btBodyEnableResponse(controlBody,0)
btBodySetActivationState(controlBody,DISABLE_DEACTIVATION)

Global control_joint=0
Global msx#,msy#
Global hide_meshes=0

MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
Repeat	
	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	msx#=MouseXSpeed()
	msy#=MouseYSpeed()

	If KeyHit(44) Then 
		phResetPendulums()
		phPendulumsApplyForce(0)
	EndIf

	If KeyHit(45) Then 
		phResetPendulums()
		phPendulumsApplyForce(1)
	EndIf
	
	If KeyHit(46) Then 
		phResetPendulums()
		phPendulumsApplyForce(2)
	EndIf
	
	If KeyHit(47) Then 
		phResetPendulums()
		phPendulumsApplyForce(3)
	EndIf	
	
	If KeyHit(35)
		If hide_meshes=0
			phHidePendulums()
			hide_meshes=1
		Else
			phShowPendulums()
			hide_meshes=0
		EndIf
	EndIf
	
	btRaySetPosition(btRay,EntityX(camera,1),EntityY(camera,1),EntityZ(camera,1))
	mouse_hit=MouseHit(1)
	
	CameraPick(camera,MouseX(),MouseY())
	If PickedEntity()<>0 And control_joint=0
		If mouse_hit Then
			btRaySetDirection(btRay,PickedX#()-EntityX(camera,1), PickedY#()-EntityY(camera,1), PickedZ#()-EntityZ(camera,1))
			btRayGetPickInfo(btRay,0)
			pickBody=btPickInfoBody()
			If pickBody<>0
				cx#=btPickInfoX()
				cy#=btPickInfoY()
				cz#=btPickInfoZ()
				btBodySetPosition(controlBody,cx,cy,cz)
				btBodySetSleep(pickBody,1)
				;btBodySetLinearVelocity(pickBody,0,0,0)
				;btBodyApplyForce(pickBody,0,0,2000)
				control_joint=btJointCreate6DofGeneric(pickBody,controlBody, cx,cy,cz,1, cx,cy,cz,1, 0)
			EndIf
		EndIf
	EndIf

	If MouseDown(1) Then
		If pickBody<>0 Then
			cx# = cx + msx/4
			cy# = cy - msy/4
			btBodySetPosition(controlBody,cx,cy,cz)
			CameraProject camera,btBodyGetX(controlBody),btBodyGetY(controlBody),btBodyGetZ(controlBody)
			MoveMouse ProjectedX(),ProjectedY()
		EndIf
	Else
		If control_joint<>0 Then
			btDeleteJoint(control_joint)
			control_joint=0
			pickBody=0
		EndIf
	EndIf

	btSimulation(.1,6)
	
	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)
	
	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	Text 10,70,"Z, X, C, V - reset pendulums and apply force"
	
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End

