;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Boat"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/60)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)


camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,100,-120
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,25,0,20

terrain=LoadTerrain( "..\media\Height_map.bmp" ) 
ScaleEntity terrain,15,100,15
PositionEntity terrain,-430,-80,-430

grass_tex=LoadTexture( "..\media\grass.jpg" )
ScaleTexture grass_tex,20,20
EntityTexture terrain,grass_tex,0,1 

bodyTerratin=btBodyCreateTerrain(terrain,15,100,15)
btBodySetPosition(bodyTerratin,50,-32.5,50)
btBodyEnableDebugDraw(bodyTerratin,0)
btBodySetCollisionGroup(bodyTerratin,2)
btCollisionGroupSetFlag(1,2,0)

meshWater=CreateCube()
ScaleEntity meshWater,800,200,800
EntityAlpha meshWater,0.5
EntityColor meshWater,50,50,150

bodyWater=btBodyCreateBox(800,200,800,0)
btBodyEnableCollision(bodyWater,0)
btBodyAttachEntity(bodyWater,meshWater)
btBodySetPosition(bodyWater,0,-200,0)
btBodySetCollisionGroup(bodyWater,1)
btTriggerSetIntersectionParam(bodyWater,1,0,0)
btIntersectionTestAddTrigger(bodyWater)

meshBoat=LoadMesh("..\media\fsaucer1.3ds")
ScaleMesh meshBoat,0.7,1.5,2

bodyBoat=btBodyCreateConvexHull(meshBoat,50,1)
btBodyAttachEntity(bodyBoat,meshBoat)
btBodySetPosition(bodyBoat,0,100,70)
btBodySetRotation(bodyBoat,0,180,0)
btBodySetAngularDamping(bodyBoat,0.25)
btBodySetCollisionGroup(bodyBoat,3)
btCollisionGroupSetFlag(2,3,1)

Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
HidePointer
Repeat	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	If KeyHit(35) Then HideShowMeshes()

	msx#=MouseXSpeed()
	msy#=MouseYSpeed()
	Camera_Look(camera)
	ActiveCamera(camera,meshBoat,meshBoat,0.03,0.03,0.03,150,100,0,0)

	btTriggersIntersectionTest(False)

	If btTriggerIsBodyIntersect(bodyWater,bodyBoat)
		force#=-btTriggerGetMaxDistance(bodyWater)*80
		btBodyApplyForce(bodyBoat,0,force,0)
	
		If btTriggerGetMaxDistance(bodyWater)>-12
			btBodySetLinearDamping(bodyBoat,0.3)
		Else
			btBodySetLinearDamping(bodyBoat,0.6)
		EndIf
		
		If KeyDown(203)
			btBodyApplyLocalTorque(bodyBoat,0,-4000,40)
		Else
			If KeyDown(205)
				btBodyApplyLocalTorque(bodyBoat,0,4000,-40)
			EndIf
		EndIf

		If KeyDown(200)
			btBodyApplyLocalForce(bodyBoat,0,0,-1000)
			btBodyApplyLocalTorque(bodyBoat,2000,0,0)
		Else
			If KeyDown(208)
				btBodyApplyLocalForce(bodyBoat,0,0,1000)
				btBodyApplyLocalTorque(bodyBoat,-2000,0,0)
			EndIf
		EndIf

		If btBodyGetRoll(bodyBoat)>4
			btBodyApplyLocalTorque(bodyBoat,0,0,-2000)
		EndIf
	
		If btBodyGetRoll(bodyBoat)<-4
			btBodyApplyLocalTorque(bodyBoat,0,0,2000)
		EndIf
	
		If btBodyGetPitch(bodyBoat)>4
			btBodyApplyLocalTorque(bodyBoat,-6000,0,0)
		EndIf
	
		If btBodyGetPitch(bodyBoat)<-4
			btBodyApplyLocalTorque(bodyBoat,6000,0,0)
		EndIf
	Else
		btBodySetLinearDamping(bodyBoat,0.1)
	EndIf

	btSimulation(.1,6)
	
	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)

	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"Arrows - movement a boat"
	
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
	Field group
End Type

Function phCreateSphere%()
	
	phEnt.phEntity = New phEntity
	radius#=Rnd(4,6)
	phEnt\mesh=CreateSphere()
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,Rnd(5,10))
			
	If hide=1
		HideEntity phEnt\mesh
	EndIf

	EntityColor phEnt\mesh,Rand(100,200),Rand(100,200),Rand(100,200)
	
	btBodySetPosition(phEnt\body,Rnd(-50,150),Rnd(50,100),Rnd(-50,50))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	
	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)

	Return phEnt\body
End Function

Function phSphereGetEntity%()
	
	phEnt.phEntity = Last phEntity
	Return phEnt\mesh
	
End Function

Function HideShowMeshes()

	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh
			HideEntity meshPlane
			HideEntity meshMainChair
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
			ShowEntity meshPlane
			ShowEntity meshMainChair
		Next
		hide=0
	EndIf
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)

	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function

Function ActiveCamera(cam,pivot_car,ent_car,cspeedX#,cspeedY#,cspeedZ#,dist#,hite#,xrot#,Modecamera)

	If KeyDown (30) Then 
		TurnEntity pivot_car,0,-1,0
	Else	
		If KeyDown (32) Then TurnEntity pivot_car,0,1,0
 	EndIf
	
	Select Modecamera	  	    
		Case 0
		    EntityParent cam,0
			PointEntity cam,pivot_car
			
			TFormPoint 0,hite,dist,pivot_car,0
			cx#=(TFormedX()-EntityX(cam))*(cspeedX#)
			cy#=(TFormedY()-EntityY(cam))*(cspeedY#)
			cz#=(TFormedZ()-EntityZ(cam))*(cspeedZ#)
			TranslateEntity cam,cx,cy,cz
			flagPosCamera=0
		Case 2		 
		    RotateEntity cam,0,0,0
			PositionEntity cam,0,35,15
			EntityParent cam,ent_car	
			flagPosCamera=0						
		Case 3
		    EntityParent cam,ent_car
			RotateEntity cam,0,0,0
			PositionEntity cam,-10,27,-4
			flagPosCamera=0	
		Case 4
			RotateEntity cam,0,0,0
			PositionEntity cam,-27,15,-2
			flagPosCamera=0	
		Case 1
			EntityParent cam,0
			If flagPosCamera=0
				PositionEntity cam,400,170,-260
				flagPosCamera=1
			EndIf
			PointEntity cam,ent_car	   	    	
	End Select 
	
End Function
