;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Static deformation"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverMode(SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,200,-450
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 150,150,150
light=CreateLight()
PositionEntity light,0,400,0
RotateEntity light,10,0,0

Global groundMesh=LoadMesh("..\media\curve_plane.b3d")
groundSurface=GetSurface(groundMesh,1)
ScaleMesh groundMesh,5,5,5
tex=LoadTexture("..\media\cell.png")
ScaleTexture tex,.1,.1
EntityTexture groundMesh,tex,0,1 

groundBody=btBodyCreateTrimesh(groundMesh,0,1)
btBodySetMargin(groundBody,0.8)
btBodySetBoundingBox(groundBody,-100,-50,100,100,50,100,1)
btBodySetContactProcessingThreshold(groundBody, 0.0)

For i=0 To 20
	phCreateSphere()
	phCreateCube()
Next


Global mx#,my#,msx#,msy#
Global hide=0
Const UPS=60
period=1000/UPS
time=MilliSecs()-period

Repeat

	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	If KeyHit(35) Then HideShowMeshes()

	;msx#=MouseXSpeed()
	;msy#=MouseYSpeed()
	;Camera_Look(camera)

	offset#=offset+1
	For i=78 To 89
		For j=0 To 6
			n=i+24*j
			vertex=TriangleVertex(groundSurface,n,0)
			vx#=VertexX(groundSurface,vertex)
			vy#=(26*Sin(Float(n)+offset)*Cos(Float(i*25)+offset))
			vz#=VertexZ(groundSurface,vertex)
			VertexCoords(groundSurface,vertex,vx,vy,vz)
			btVertexSetPosition(groundBody,n,0,vx,vy,vz,0)
		Next
	Next
	btBodyUpdateBound(groundBody)
	UpdateWorld
	btSimulation(0.09,6)
	
	RenderWorld()
	btDebugDrawWorld(camera)
	Frames=Frames+1  
   	If MilliSecs()-render_time=>1000  fps1=Frames : Frames=0 : render_time=MilliSecs()
	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	
	Flip 1
	
Until KeyHit(1)
btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
End Type

Function phCreateSphere()
	phEnt.phEntity = New phEntity
	radius#=Rnd(12,15)
	phEnt\mesh=CreateSphere()
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,Rnd(5,10))
	EntityColor phEnt\mesh,Rand(100,160),Rand(100,160),Rand(100,160)		
	btBodySetPosition(phEnt\body,Rnd(-250,250),Rnd(50,290),Rnd(-250,250))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.02)
	btBodySetLinearDamping(phEnt\body,0.2)
	
	btBodySetAngularSleepingThreshold(phEnt\body,0.6)
	btBodySetLinearSleepingThreshold(phEnt\body,0.6)
End Function

Function phCreateCube()
	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateCube()
	EntityShininess phEnt\mesh,0.5
	dx#=Rnd(12,14)
	dy#=Rnd(12,14)
	dz#=Rnd(12,14)
	ScaleEntity phEnt\mesh,dx,dy,dz
	phEnt\body=btBodyCreateBox(dx,dy,dz,Rnd(10,20))
	EntityColor phEnt\mesh,Rand(100,160),Rand(100,160),Rand(100,160)		
	btBodySetPosition(phEnt\body,Rnd(-250,250),Rnd(50,290),Rnd(-250,250))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.02)
	btBodySetLinearDamping(phEnt\body,0.2)
	
	btBodySetAngularSleepingThreshold(phEnt\body,0.08)
	btBodySetLinearSleepingThreshold(phEnt\body,0.06)
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)
	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function

Function HideShowMeshes()
	
	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh
			HideEntity groundMesh		
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
			ShowEntity groundMesh
		Next
		hide=0
	EndIf
	
End Function
