;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Dynamic deformation"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/90)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)
btContactSetSolverIterations(20)

camera=CreateCamera()
AmbientLight 40,40,40
PositionEntity camera,0,100,-100
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 150,150,150
light=CreateLight()
RotateEntity light,50,0,70
PositionEntity light,100,50,0

Global meshPlane=CreatePlane()
Global tex=LoadTexture("..\media\cell.png")
Global tex_mesh=LoadTexture("..\media\cell.png")
ScaleTexture tex,10,10
EntityTexture meshPlane,tex,0,1 

btBodyCreatePlane(0,1,0)

meshBarrel=LoadMesh("..\media\barrel.b3d")

bodyBarrel=btBodyCreateTrimesh(meshBarrel,200,TRIMESH_CONCAVE)
btBodySetPosition(bodyBarrel,0,50,0)
btBodySetMargin(bodyBarrel,0.7)
btBodyAttachEntity(bodyBarrel,meshBarrel)
btBodySetUserData(bodyBarrel,meshBarrel)

btRay=btCreateRay()
btRaySetMaxDistance(btRay,10)

body_shot=0
pickBody=0
Global mx#,my#,msx#,msy#
Global hide=0
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
HidePointer
Repeat	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1+DBG_RAYS+DBG_RAYCASTPOINTS)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	msx#=MouseXSpeed()
	msy#=MouseYSpeed()
	Camera_Look(camera)

	If MouseHit(1)
		body_shot=phCreateSphere()
		TFormVector (0,0,1,camera,0)
		btBodySetPosition body_shot,EntityX(camera)+TFormedX()*10,EntityY(camera)+TFormedY()*10,EntityZ(camera)+TFormedZ()*10
		btBodyApplyImpulse(body_shot,TFormedX()*1000, TFormedY()*1000, TFormedZ()*1000)
	EndIf

	If body_shot<>0 Then
		btContactTestAddBody(body_shot,0)	
		btContactManifoldTest(True)
		For i=0 To btContactGetNum(body_shot)-1
			If btBodyGetLinearVelocity(body_shot)>9 Then
				btRaySetPosition(btRay,btBodyGetX(body_shot),btBodyGetY(body_shot),btBodyGetZ(body_shot))
				nx#=btContactGetNX(body_shot,i)
				ny#=btContactGetNY(body_shot,i)
				nz#=btContactGetNZ(body_shot,i)

				btRaySetDirection(btRay,nx,ny,nz)	
				If btRayGetPickInfo(btRay,1)<>0 Then
					pickBody=btPickInfoBody()
					triangle=btPickInfoTriangle()
					If triangle<>-1
						For i=0 To 2
							btVertexTranslate(btPickInfoBody(),triangle,i,nx*Rnd(2,3),ny*Rnd(2,3),nz*Rnd(2,3),1)
						
							vx#=btVertexGetX(btPickInfoBody(),triangle,i,0)
							vy#=btVertexGetY(btPickInfoBody(),triangle,i,0)
							vz#=btVertexGetZ(btPickInfoBody(),triangle,i,0)

							mesh=btBodyGetUserData(bodyBarrel)
							vertex=TriangleVertex(GetSurface(mesh,1),triangle,i)
							VertexCoords(GetSurface(mesh,1),vertex,vx,vy,vz)
						Next
					EndIf
				EndIf
			EndIf
		Next
	EndIf

	btSimulation(.3,9)
	
	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)

	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"Left mouse - to throw sphere"
	Text 10,70,"W,A,S,D - camera movement"
	
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
End Type

Function phCreateSphere%()
	
	phEnt.phEntity = New phEntity
	radius#=Rnd(4,6)
	phEnt\mesh=CreateSphere()
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,Rnd(5,10))
			
	If hide=1
		HideEntity phEnt\mesh
	EndIf

	EntityColor phEnt\mesh,Rand(100,200),Rand(100,200),Rand(100,200)
	
	btBodySetPosition(phEnt\body,Rnd(-50,150),Rnd(50,100),Rnd(-50,50))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	
	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)

	Return phEnt\body
End Function

Function phSphereGetEntity%()
	
	phEnt.phEntity = Last phEntity
	Return phEnt\mesh
	
End Function

Function HideShowMeshes()

	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh
			HideEntity meshPlane
			HideEntity meshMainChair
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
			ShowEntity meshPlane
			ShowEntity meshMainChair
		Next
		hide=0
	EndIf
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)

	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function

Function ActiveCamera(cam,pivot_car,ent_car,cspeedX#,cspeedY#,cspeedZ#,dist#,hite#,xrot#,Modecamera)

	If KeyDown (30) Then 
		TurnEntity pivot_car,0,-1,0
	Else	
		If KeyDown (32) Then TurnEntity pivot_car,0,1,0
 	EndIf
	
	Select Modecamera	  	    
		Case 0
		    EntityParent cam,0
			PointEntity cam,pivot_car
			
			TFormPoint 0,hite,dist,pivot_car,0
			cx#=(TFormedX()-EntityX(cam))*(cspeedX#)
			cy#=(TFormedY()-EntityY(cam))*(cspeedY#)
			cz#=(TFormedZ()-EntityZ(cam))*(cspeedZ#)
			TranslateEntity cam,cx,cy,cz
			flagPosCamera=0
		Case 2		 
		    RotateEntity cam,0,0,0
			PositionEntity cam,0,35,15
			EntityParent cam,ent_car	
			flagPosCamera=0						
		Case 3
		    EntityParent cam,ent_car
			RotateEntity cam,0,0,0
			PositionEntity cam,-10,27,-4
			flagPosCamera=0	
		Case 4
			RotateEntity cam,0,0,0
			PositionEntity cam,-27,15,-2
			flagPosCamera=0	
		Case 1
			EntityParent cam,0
			If flagPosCamera=0
				PositionEntity cam,400,170,-260
				flagPosCamera=1
			EndIf
			PointEntity cam,ent_car	   	    	
	End Select 
	
End Function
