;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800,600,32,2
SetBuffer(BackBuffer())
AppTitle "BlitzBullet: Soft body - Car"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")

Global Frames,LastCheck,FPS_
Global mx#,my#,msx#,msy#

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/50)
btDebugDrawEnableNodeTree(0)

light=CreateLight()
AmbientLight 90,90,90
RotateEntity light,20,0,0
PositionEntity light,0,100,-400

camera=CreateCamera()
AmbientLight 180,180,180
CameraRange camera,1,15000
PositionEntity camera,0,100,-300
RotateEntity camera,10,0,0
CameraClsColor  camera,40,40,40

ground=CreateCube()
ScaleMesh ground,1550,1,1200
EntityColor ground,120,120,120
texCell=LoadTexture("..\media\cell.png")
ScaleTexture texCell,0.05,0.05
EntityTexture ground,texCell


bodyGround=btBodyCreateConvexHull(ground,0,0)
btBodySetPosition(bodyGround,-800,0,0)
btBodyAttachEntity(bodyGround,ground)

;Create box stairs
For i=0 To 34
	c=CreateCube()
	ScaleEntity c,15,15,300
	PositionEntity c,250+(i*30),15+(i*15),0
	EntityColor c,Rnd(50,100),Rnd(50,100),Rnd(50,100)
	bc=btBodyCreateBox(15,15,300,0)
	btBodySetPosition(bc,250+(i*30),15+(i*15),0)
Next
For i=0 To 10
	c=CreateCube()
	dx#=Rnd(20,25)
	dy#=Rnd(7,12)
	dz#=Rnd(70,125)
	ScaleEntity c,dx,dy,dz
	EntityColor c,Rnd(50,100),Rnd(50,100),Rnd(50,100)
	bc=btBodyCreateBox(dx,dy,dz,Rnd(250,200))
	btBodySetPosition(bc,Rnd(-1200,200),15+(i*8),Rnd(70,-70))
	btBodyAttachEntity(bc,c)
Next

;Load meshes 
Dim aMeshWheel(4)
For i=0 To 3
	aMeshWheel(i)=LoadMesh("..\media\torus.b3d")
	tex=LoadTexture("..\media\metal.png") 
	EntityTexture aMeshWheel(i),tex 
	ScaleMesh aMeshWheel(i),0.8,.95,.8
	MeshCullBox aMeshWheel(i),-50,-50,-50,100,100,100
Next

;Create soft body
Dim aSBodyWheel(4)
Dim aSBNodeArray(5)

For i=0 To 3
	aSBodyWheel(i)=btSoftBodyCreateFromMesh(aMeshWheel(i),100,-1)
	btSoftBodyGenerateClusters(aSBodyWheel(i),32)
	btSoftBodySetLinearStiffness(aSBodyWheel(i),1)
	btSoftBodySetVolumeStiffness(aSBodyWheel(i),1)
	btBodyAttachEntity(aSBodyWheel(i),aMeshWheel(i))
	btBodySetRotation(aSBodyWheel(i),0,90,90)
	btSoftBodySetPressure(aSBodyWheel(i),100000)
	
	btSoftBodySetPose(aSBodyWheel(i),False,True)
	btSoftBodySetPoseMatching(aSBodyWheel(i),0.004)
Next

btBodySetPosition(aSBodyWheel(0),1020,620,60)
btBodySetPosition(aSBodyWheel(1),1020,620,-60)
btBodySetPosition(aSBodyWheel(2),1090,620,60)
btBodySetPosition(aSBodyWheel(3),1090,620,-60)

For i=0 To 3
	aSBNodeArray(i)=btSoftBodyCreateArrayNodes(aSBodyWheel(i))
Next

mesh=LoadMesh("..\media\car.b3d")
ScaleMesh mesh,2,2,2
MeshCullBox mesh,-50,-50,-50,100,100,100
tex=LoadTexture ("..\media\color4.jpg")
EntityTexture mesh,tex
EntityShininess mesh,0.4
sbody=btSoftBodyCreateFromMesh(mesh,400,2)
btSoftBodySetDamping(sbody,.0004)
btSoftBodyGenerateClusters(sbody,32)
btSoftBodySetLinearStiffness(sbody,0.1)
btSoftBodySetAngularStiffness(sbody,0.1)
btSoftBodySetVolumeStiffness(sbody,0.1)
btBodySetRotation(sbody,0,90,0)
btBodySetPosition(sbody,1140,760,-22)
btBodyAttachEntity(sbody, mesh)
aSBNodeArray(4)=btSoftBodyCreateArrayNodes(sbody)

jointLfr=btSoftBodyAppendLJoint(sbody, aSBodyWheel(0),0.4*170,-0.54*100,0.71*120)
btSoftBodyJointSetDirAxis(jointLfr,0,1,0,1)

jointLfl=btSoftBodyAppendLJoint(sbody, aSBodyWheel(1),-0.4*170,-0.54*100,0.71*120)
btSoftBodyJointSetDirAxis(jointLfl,0,1,0,1)

jointLbr=btSoftBodyAppendLJoint(sbody, aSBodyWheel(2),0.4*170,-0.54*100,-0.71*120)
btSoftBodyJointSetDirAxis(jointLbr,0,1,0,1)

jointLbl=btSoftBodyAppendLJoint(sbody, aSBodyWheel(3),-0.4*170,-0.54*100,-0.71*120)
btSoftBodyJointSetDirAxis(jointLbl,0,1,0,1)

jointAfr=btSoftBodyAppendAJoint(sbody,aSBodyWheel(0),0,-1,0)
btSoftBodyJointSetDirAxis(jointAfr,1,0,0,0)

jointAfl=btSoftBodyAppendAJoint(sbody,aSBodyWheel(1),0,-1,0)
btSoftBodyJointSetDirAxis(jointAfl,1,0,0,0)

jointAbr=btSoftBodyAppendAJoint(sbody,aSBodyWheel(2),0,-1,0)
btSoftBodyJointSetDirAxis(jointAbr,1,0,0,0)

jointAbl=btSoftBodyAppendAJoint(sbody,aSBodyWheel(3),0,-1,0)
btSoftBodyJointSetDirAxis(jointAbl,1,0,0,0)


cameraPivot=CreatePivot()
EntityParent camera,cameraPivot

Const UPS=60
period=1000/UPS
time=MilliSecs()-period
isPhysicsUpdate=1
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
HidePointer 

Repeat
	Repeat
		elapsed=MilliSecs()-time
	Until elapsed
	ticks=elapsed/period
	tween#=Float(elapsed Mod period)/Float(period)
	
	For k=1 To ticks
		time=time+period
		
		msx#=MouseXSpeed()
		msy#=MouseYSpeed()
		
		Camera_Look(camera)

		If KeyHit(2) btDebugDrawSetMode(0)
		If KeyHit(3) btDebugDrawSetMode(1)
		If KeyHit(4) btDebugDrawSetMode(2)
		If KeyHit(5) btDebugDrawSetMode(1+2)

		If KeyHit(57)
			If isPhysicsUpdate=1
				isPhysicsUpdate=0
			Else
				isPhysicsUpdate=1
			EndIf
		EndIf

		If KeyHit(28)
			For i=0 To 3
				btSoftBodySetArrayNodes(aSBNodeArray(i))
			Next
			btSoftBodySetArrayNodes(aSBNodeArray(4))
		EndIf

		If KeyDown(200)
			btBodyApplyForce(sbody,-40,0,0)
		Else
			If KeyDown(208)
				btBodyApplyForce(sbody,40,0,0)
			EndIf
		EndIf

		PositionEntity cameraPivot,btBodyGetX(sbody),btBodyGetY(sbody),btBodyGetZ(sbody)
		
		If isPhysicsUpdate=1
			btSimulation(0.06,3)
		EndIf
		UpdateWorld
	Next

	RenderWorld()
	btDebugDrawWorld(camera)
	Frames=Frames+1  
   	If MilliSecs()-render_time=>1000  fps1=Frames : Frames=0 : render_time=MilliSecs()  
	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"Enter - reset car"
	Text 10,70,"Space - stop simulation"
	Text 10,90,"W,A,S,D - move camera"
	Text 10,110,"Up, Down - apply force to car"
	
	Flip 0
	
Until KeyHit(1)
btDestroyWorld()
End

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)
	If KeyDown(17) Then 
		MoveEntity cam,0,0,4
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-4
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-4,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,4,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function
