;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Soft body - Cloth"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btContactSetSolverIterations(10)

camera=CreateCamera()
CameraClsColor camera,20,20,20
CameraRange  camera,1,2500
AmbientLight 140,140,140
PositionEntity camera,0,150,-180
RotateEntity camera,25,0,0

light=CreateLight(1)
PositionEntity light,-50,150,-260
RotateEntity  light,0,50,0

texCell=LoadTexture("..\media\cell.png")
ScaleTexture texCell,.2,.2
plane=CreatePlane()
ScaleEntity plane,200,0.5,200
EntityTexture  plane,texCell

bodyPlane=btBodyCreatePlane(0,1,0)
btBodySetFriction(bodyPlane,0.08)
btBodySetContactProcessingThreshold(bodyPlane,0.02)
btBodySetMargin(bodyPlane,0.2)

Global meshCloth=LoadMesh("..\media\plane2.b3d")
MeshCullBox meshCloth,-50,-50,-50,200,200,200
texLogo=LoadTexture ("..\media\logo.png")
ScaleTexture texLogo,1,.2
EntityTexture meshCloth,texLogo
EntityFX meshCloth,16

softbodyCloth=btSoftBodyCreateFromMesh(meshCloth,16,2)
btBodySetRotation(softbodyCloth,-90,0,0)
btBodySetPosition(softbodyCloth,0,70,0)
btBodyAttachEntity(softbodyCloth,meshCloth)
btSoftBodySetFlagCollisions(softbodyCloth,2)

btSoftBodySetMass(softbodyCloth,0,343)
btSoftBodySetMass(softbodyCloth,0,360)
btSoftBodyGenerateClusters(softbodyCloth,64)
btSoftBodySetClusterRigidHardness(softbodyCloth,0.1)

btBodySetMargin(softbodyCloth,2)
btSoftBodySetDamping(softbodyCloth,0.001)
btSoftBodySetLinearStiffness(softbodyCloth,0.01)
btSoftBodySetDynamicFriction(softbodyCloth,0.1) 


sphere1=phCreateSphere()
btBodySetPosition(sphere1,50,60,-40)
sphere2=phCreateSphere()
btBodySetPosition(sphere2,-50,60,-40)

an1=btSoftBodyAppendAnchor(softbodyCloth,37,sphere1,1)
btSoftBodyAnchorSetLocalPosition(an1,0,0,8)

an2=btSoftBodyAppendAnchor(softbodyCloth,1,sphere2,1)
btSoftBodyAnchorSetLocalPosition(an2,0,0,8)

btSoftBodySetPose(softbodyCloth,False,1)
btSoftBodySetRigidContactHardness(softbodyCloth,2)

Global hide=0
Global mx#,my#,msx#,msy#
Const UPS=60
period=1000/UPS
time=MilliSecs()-period
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
HidePointer 

Repeat

	Repeat
		elapsed=MilliSecs()-time
	Until elapsed
	ticks=elapsed/period
	
	For k=1 To ticks
		time=time+period

		If KeyHit(2) btDebugDrawSetMode(0)
		If KeyHit(3) btDebugDrawSetMode(1)
		If KeyHit(4) btDebugDrawSetMode(2)
		If KeyHit(5) btDebugDrawSetMode(1+2)

		If KeyHit(35) Then HideShowMeshes()

		msx#=MouseXSpeed()
		msy#=MouseYSpeed()
		Camera_Look(camera)

		If MouseHit(1)
			sphere=phCreateSphere()
			TFormVector (0,0,1,camera,0)
			btBodySetPosition sphere,EntityX(camera)+TFormedX()*10,EntityY(camera)+TFormedY()*10,EntityZ(camera)+TFormedZ()*10
			btBodyApplyImpulse(sphere,TFormedX()*1000, TFormedY()*1000, TFormedZ()*1000)
		EndIf
		
		btSimulation(0.2,4)
		UpdateWorld
	Next
	
	RenderWorld()
	btDebugDrawWorld(camera)

	Frames=Frames+1  
   	If MilliSecs()-render_time=>1000  fps1=Frames : Frames=0 : render_time=MilliSecs()  
	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	Text 10,70,"W,A,S,D - move camera"
	Text 10,90,"Left mouse - to throw sphere"	
	
	Flip 1
	
Until KeyHit(1)
btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
End Type

Function phCreateSphere%()
	
	phEnt.phEntity = New phEntity
	radius#=Rnd(7,9)
	phEnt\mesh=CreateSphere()
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,radius,radius,radius
	phEnt\body=btBodyCreateSphere(radius,Rnd(5,10))
			
	If hide=1
		HideEntity phEnt\mesh
	EndIf

	EntityColor phEnt\mesh,Rand(100,200),Rand(100,200),Rand(100,200)
	
	btBodySetPosition(phEnt\body,Rnd(-50,150),Rnd(50,100),Rnd(-50,50))
	btBodySetRotation(phEnt\body,Rnd(-180,180),Rnd(-180,180),Rnd(-180,180))
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	
	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)

	Return phEnt\body
End Function

Function HideShowMeshes()
	
	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh
		Next
		HideEntity meshCloth		
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
		Next
		ShowEntity meshCloth
		hide=0
	EndIf
	
End Function

Function Camera_Look(cam,speedx#=.5,speedy#=.5,dang=-89,upang=89)
	If KeyDown(17) Then 
		MoveEntity cam,0,0,1
	Else
		If KeyDown(31) Then MoveEntity cam,0,0,-1
	EndIf
	If KeyDown(30) Then 
		MoveEntity cam,-1,0,0	
	Else
		If KeyDown(32) Then MoveEntity cam,1,0,0
	EndIf	
	
	xrot#=EntityPitch(cam)
	yrot#=EntityYaw(cam)
	If(xrot#<upang)And(msy>0)Then xrot#=xrot#+speedx#*msy/10  
	If(xrot#>upang)And(msy>0)Then xrot#=upang	
	If(xrot#>dang)And(msy<0)Then xrot#=xrot#+speedx#*msy/10 
	If(xrot#<dang)And(msy<0)Then xrot#=dang		
	yrot=yrot-speedy#*msx/10 
	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	RotateEntity cam,xrot,yrot,0
End Function