;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Multi Material"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")
SeedRnd MilliSecs ()

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(0)
btWorldSetFixedTimeStep(1.0/30)
btContactSetSolverMode(SOLVER_USE_2_FRICTION_DIRECTIONS+SOLVER_SIMD+SOLVER_RANDMIZE_ORDER)

camera=CreateCamera()
AmbientLight 180,180,180
PositionEntity camera,0,100,-130
RotateEntity camera,20,0,0
CameraClsColor camera,70,70,70

AmbientLight 90,90,90
light=CreateLight()
RotateEntity light,25,0,20

Global meshPlane=CreatePlane()
texPlane=LoadTexture( "..\media\cell.png" )
ScaleTexture texPlane,60,60
EntityTexture meshPlane,texPlane,0,1 

bodyPlane=btBodyCreatePlane(0,1,0)

tex=LoadTexture("..\media\cell_grey.png")
ScaleTexture tex,0.1,0.1
planeMultiMaterial=LoadMesh("..\media\planeMultiMaterial.b3d")
EntityTexture planeMultiMaterial,tex
ScaleMesh planeMultiMaterial,1.5,1,1.5
EntityFX planeMultiMaterial,16

materials=btCreateMaterials(2)

btMaterialsSetParam(materials,0,0.4,0.2)
btMaterialsSetTriangles(materials,0,0,143)
btMaterialsSetParam(materials,1,10,2)
btMaterialsSetTriangles(materials,1,144,288)

bodyMultiMaterial=btBodyCreateMultiMaterialTrimesh(planeMultiMaterial,0,materials)
btBodyAttachEntity(bodyMultiMaterial,planeMultiMaterial)
btBodySetPosition(bodyMultiMaterial,0,40,0)
btBodySetRotation(bodyMultiMaterial,0,0,-15)

body1=phCreateCube(155,0,0)
btBodySetPosition(body1,-50,80,40)
btBodySetRotation(body1,0,0,-10)

body2=phCreateCube(155,0,0)
btBodySetPosition(body2,-50,80,-40)
btBodySetRotation(body2,0,0,-10)

Repeat	
	If KeyHit(2) btDebugDrawSetMode(0)
	If KeyHit(3) btDebugDrawSetMode(1)
	If KeyHit(4) btDebugDrawSetMode(2)
	If KeyHit(5) btDebugDrawSetMode(1+2)

	btSimulation(.06,6)

	If KeyHit(57) Then
		btBodySetPosition(body1,-50,80,40)
		btBodySetRotation(body1,0,0,-10)
		btBodySetSleep(body1,1)
		btBodyClearForces(body1)
		
		btBodySetPosition(body2,-50,80,-40)
		btBodySetRotation(body2,0,0,-10)
		btBodySetSleep(body2,1)
		btBodyClearForces(body2)
	EndIf

	UpdateWorld()
	RenderWorld()
	btDebugDrawWorld(camera)

	frames=frames+1
   	If MilliSecs()-render_time=>1000  fps1=frames : frames=0 : render_time=MilliSecs()  
 	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"Space - reset scene"
	
	Flip 1
	
Until KeyHit(1)

btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
End Type

Function phCreateCube%(r%, g%, b%)
	phEnt.phEntity = New phEntity
	phEnt\mesh=CreateCube()
	dx#=4
	dy#=4
	dz#=4
	ScaleEntity phEnt\mesh,dx,dy,dz
	phEnt\body=btBodyCreateBox(dx,dy,dz,Rnd(10,20))
	EntityColor phEnt\mesh,r,g,b		
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	btBodySetFriction(phEnt\body,0.1)
	btBodySetLinearDamping(phEnt\body,0.05)
	
	btBodySetAngularSleepingThreshold(phEnt\body,0.08)
	btBodySetLinearSleepingThreshold(phEnt\body,0.06)
	
	Return phEnt\body
End Function
