;*******************************************************************
;*******************************************************************
;***                                        
;***	Example of use BlitzBullet 
;***	(Physics Bullet Library wrapper for Blitz3D) 
;***   
;***	(c) 2010-2011 Mirage-lab Team 
;***	www.tools.mirage-lab.com                       
;***                                                  
;*******************************************************************
;*******************************************************************

Include "..\include\BlitzBullet.bb"

Graphics3D 800, 600,32,2
AppTitle "BlitzBullet: Raycast - Masking"
SetBuffer(BackBuffer())
SetFont LoadFont("Blitz")

btCreateWorld(0,"license_key")
btCreateDebugDraw()
btDebugDrawSetMode(1+DBG_RAYS+DBG_RAYCASTPOINTS)
btDebugDrawSetSizePointRays(4)

camera=CreateCamera()
CameraClsColor camera,20,20,20
CameraRange  camera,1,2500
AmbientLight 140,140,140
PositionEntity camera,0,100,-120
RotateEntity camera,25,0,0

light=CreateLight(1)
PositionEntity light,-50,150,-260
RotateEntity  light,0,50,0

texCell=LoadTexture("..\media\cell.png")
ScaleTexture texCell,.125,.125
plane=CreatePlane()
EntityColor plane,140,140,140
ScaleEntity plane,200,0.5,200
EntityTexture  plane,texCell

Global hide=1
bodyPlane=btBodyCreatePlane(0,1,0)
btBodyEnableDebugDraw(bodyPlane,False)


IndexAdditionalGroup=0 ; << possible values: 0 - 15; if less than zero then use groups of collision
			     ;  : 0 - 15;   ,     
cube=phCreateCube()	
btBodySetPosition(cube,-75,50,0)
If IndexAdditionalCGroup>=0 Then
	btBodySetAdditionalGroup(cube,IndexAdditionalGroup,1)
EndIf

cube=phCreateCube()
btBodySetColorName(cube,0,0,0)
btBodySetPosition(cube,-25,50,0)
If IndexAdditionalCGroup>=0 Then
	btBodySetAdditionalGroup(cube,IndexAdditionalGroup,2)
EndIf

cube=phCreateCube()
btBodySetPosition(cube,25,50,0)
If IndexAdditionalCGroup>=0 Then
	btBodySetAdditionalGroup(cube,IndexAdditionalGroup,3)
EndIf

cube=phCreateCube()
btBodySetPosition(cube,75,50,0)
If IndexAdditionalCGroup>=0 Then
	btBodySetAdditionalGroup(cube,IndexAdditionalGroup,4)
EndIf

For phEnt.phEntity = Each phEntity
	btBodySetAngularDamping(phEnt\body,1)
Next

ray=btCreateRay()
btRaySetPosition(ray,100,15,0)
btRaySetDirection(ray,-1,0,0)
btRaySetMaxDistance(ray,200)
btRaySetMethodPick(ray,1)
btRaySetIndexGroup(ray,IndexAdditionalGroup)

mask=btCreateMask()
btRaySetMask(ray,mask)

Global mx#,my#,msx#,msy#
Const UPS=60
period=1000/UPS
time=MilliSecs()-period
Repeat

	Repeat
		elapsed=MilliSecs()-time
	Until elapsed
	ticks=elapsed/period
	
	For k=1 To ticks
		time=time+period

		If KeyHit(2) btDebugDrawSetMode(DBG_RAYS+DBG_RAYCASTPOINTS)
		If KeyHit(3) btDebugDrawSetMode(1+DBG_RAYS+DBG_RAYCASTPOINTS)
		If KeyHit(4) btDebugDrawSetMode(2+DBG_RAYS+DBG_RAYCASTPOINTS)
		If KeyHit(5) btDebugDrawSetMode(1+2+DBG_RAYS+DBG_RAYCASTPOINTS)

		If KeyHit(35) Then HideShowMeshes()

		For i=1 To 4
			If KeyHit(43+i) Then
				If btMaskIsAddedGroup(mask,i)
					btMaskRemoveGroup(mask,i)
				Else
					btMaskAddGroup(mask,i)
				EndIf
			EndIf
		Next
		If KeyHit(57) Then btMaskClear(mask)

		btRayGetPickInfo(ray,1)
		For i=0 To btPickInfoCount()-1
			btBodyApplyForce(btPickInfoBody(ray,i),0,100,0)
		Next
		
		btSimulation(0.2,4)
		UpdateWorld
	Next
	
	RenderWorld()
	btDebugDrawWorld(camera)

	Frames=Frames+1  
   	If MilliSecs()-render_time=>1000  fps1=Frames : Frames=0 : render_time=MilliSecs()  
	Text 10,10,"FPS: "+fps1
	Text 10,30,"1,2,3,4 - debug draw mode"
	Text 10,50,"H - hide/show meshes"
	Text 10,70,"Z,X,C,V - adds/removes group to mask"
	Text 10,90,"Space - clear mask"
	
	Flip 1
	
Until KeyHit(1)
btDestroyWorld()
End

Type phEntity
	Field mesh
	Field body
End Type

Function phCreateCube%()
	
	phEnt.phEntity = New phEntity
	dx#=10
	phEnt\mesh=CreateCube()
	EntityShininess phEnt\mesh,0.5
	ScaleEntity phEnt\mesh,dx,dx,dx
	phEnt\body=btBodyCreateBox(dx,dx,dx,Rnd(5,10))
			
	If hide=1
		HideEntity phEnt\mesh
	EndIf

	EntityColor phEnt\mesh,Rand(100,200),Rand(100,200),Rand(100,200)
	btBodyAttachEntity(phEnt\body,phEnt\mesh)
	
	btBodySetAngularSleepingThreshold(phEnt\body,1)
	btBodySetLinearSleepingThreshold(phEnt\body,1)

	Return phEnt\body
End Function

Function HideShowMeshes()
	
	If hide=0 Then
		For phEnt.phEntity = Each phEntity
			HideEntity phEnt\mesh	
		Next
		hide=1
	Else
		For phEnt.phEntity = Each phEntity
			ShowEntity phEnt\mesh
		Next
		hide=0
	EndIf
	
End Function
